from huobi.client.account import AccountClient
from huobi.client.coin_margined_swap import CoinMarginedSwapClient
from huobi.client.future import FutureClient
from huobi.constant.definition import AccountType
from common.dict_util import remove_key
from common.log import create_logger

logger = create_logger("wallet")


class Wallet:
    def __init__(self, key: str, secret: str):
        params = {"api_key": key, "secret_key": secret, "init_log": True}
        self.account_client = AccountClient(**params)
        self.coin_margined_swap_client = CoinMarginedSwapClient(**params)
        self.future_client = FutureClient(**params)
        self.accounts = None

    def get_account_id(self, account_type: AccountType) -> int:
        if self.accounts is None:
            accounts = self.account_client.get_accounts()
            self.accounts = {account.type: account for account in accounts}
        return self.accounts[account_type].id

    def get_spot_balance(self, asset: str = None) -> dict:
        """
        现货账户余额
        """
        resp = self.account_client.get_balance(self.get_account_id(AccountType.SPOT))
        balances = {item.currency: item for item in resp if item.balance != '0'}
        if asset:
            return balances.get(asset)
        return balances

    def get_coin_margined_swap_balance_valuation(self, valuation_asset: str = "BTC"):
        """
        币本位账户资产估值
        """
        resp = self.coin_margined_swap_client.get_balance_valuation(valuation_asset)
        return resp

    def get_coin_margined_swap_position(self, contract_code: str = None):
        """
        币本位账户持仓
        contract_code	false	string	合约代码		支持大小写，"BTC-USD"... ,如果缺省，默认返回所有合约
        """
        return self.coin_margined_swap_client.get_swap_position_info(contract_code)

    def get_coin_margined_swap_account_info(self, contract_code: str = None):
        """
        币本位账户信息
        contract_code	false	string	支持大小写, "BTC-USD"... ,如果缺省，默认返回所有合约
        """
        return self.coin_margined_swap_client.get_swap_account_info(contract_code)

    def get_contract_balance_valuation(self, valuation_asset='BTC'):
        """
        交割合约账户估值
        valuation_asset	false	string	资产估值币种，即按该币种为单位进行估值，不填默认"BTC"	"BTC","USD","CNY","EUR","GBP","VND","HKD","TWD","MYR","SGD","KRW","RUB","TRY"
        """
        resp = self.future_client.get_contract_balance_valuation(valuation_asset)
        logger.info(resp)
        return resp

    def get_contract_account_info(self, symbol=None):
        """
        获取交割合约账户信息
        symbol	false	string	品种代码		支持大小写,"BTC","ETH"...如果缺省，默认返回所有品种
        """
        return self.future_client.get_contract_account_info(**remove_key(locals()))

    def get_contract_position_info(self, symbol=None):
        """
        获取交割合约账户持仓信息
        symbol	false	string	品种代码		支持大小写,""BTC","ETH"...如果缺省，默认返回所有品种
        备注:
        如果有某个品种在结算中，不带请求参数去查询持仓，
        会返回错误码1080(1080 In settlement or delivery. Unable to get positions of some contracts. )。
        建议您带上请求参数去查询持仓，避免报错查询不到持仓。
        """
        return self.future_client.get_contract_position_info(**remove_key(locals()))





import logging

from binance.client import AsyncClient, BaseClient
from .constant import TradeType


TransferTypeDict = {
    (TradeType.SPOT, TradeType.COIN_M): AsyncClient.SPOT_TO_COIN_FUTURE,
    (TradeType.SPOT, TradeType.USD_M): AsyncClient.SPOT_TO_USDT_FUTURE,
    (TradeType.COIN_M, TradeType.SPOT): AsyncClient.COIN_FUTURE_TO_SPOT,
    (TradeType.USD_M, TradeType.SPOT): AsyncClient.USDT_FUTURE_TO_SPOT,
}


class BinanceService:
    def __init__(self, client: AsyncClient):
        self.client = client

    async def account_info(self, trade_type, symbol: str) -> (float, float):
        """
        返回total, available
        """
        if trade_type == TradeType.SPOT:
            rst = self.client.get_account()
            if symbol:
                balance_info = [item for item in rst['balances'] if item['asset'] == symbol]
                if not balance_info:
                    return 0, 0
                return float(balance_info[0]['free']) + float(balance_info[0]['locked']), float(balance_info[0]['free'])
        elif trade_type == TradeType.COIN_M:
            rst = self.client.futures_coin_account()
            balance_info = [item for item in rst['assets'] if item['asset'] == symbol]
            if not balance_info:
                return 0, 0
            return float(balance_info[0]['walletBalance']), float(balance_info[0]['maxWithdrawAmount'])
        else:
            rst = self.client.futures_account()
            balance_info = [item for item in rst['assets'] if item['asset'] == symbol]
            if not balance_info:
                return 0, 0
            return float(balance_info[0]['walletBalance']), float(balance_info[0]['maxWithdrawAmount'])

    async def transfer(self, symbol, from_type, to_type, amount):
        """
        资金划转
        如果不涉及币币账户，都需要币币作为中转
        """
        assert from_type != to_type

        if from_type == TradeType.SPOT:
            return await self.client.universal_transfer(
                type=TransferTypeDict[(from_type, to_type)], asset=symbol, amount=str(amount)
            )
        elif from_type == TradeType.COIN_M:
            if to_type == TradeType.SPOT:
                return await self.client.universal_transfer(
                    type=TransferTypeDict[(from_type, to_type)], asset=symbol, amount=str(amount)
                )
            else:
                await self.transfer(symbol, from_type, TradeType.SPOT, amount)
                return await self.transfer(symbol, TradeType.SPOT, to_type, amount)
        elif from_type == TradeType.USD_M:
            if to_type == TradeType.SPOT:
                return await self.client.universal_transfer(
                    type=TransferTypeDict[(from_type, to_type)], asset=symbol, amount=str(amount)
                )
            else:
                await self.transfer(symbol, from_type, TradeType.SPOT, amount)
                return await self.transfer(symbol, TradeType.SPOT, to_type, amount)

    async def position_info(self, trade_type, symbol):
        data = None
        if trade_type == TradeType.COIN_M:
            rst = await self.client.futures_coin_position_information(pair=symbol)
            data = [item for item in rst if item['symbol'] == symbol]
        elif trade_type == TradeType.USD_M:
            rst = await self.client.futures_position_information(symbol=symbol)
            data = [item for item in rst if item['symbol'] == symbol]

        if not data:
            return None
        else:
            return data[0]

    async def close_all_position(self, trade_type, symbol):
        position_info = await self.position_info(trade_type, symbol)
        amount = int(position_info['positionAmt'])
        if amount > 0:
            side = BaseClient.SIDE_SELL
        else:
            side = BaseClient.SIDE_BUY
        params = {
            "symbol": symbol,
            "side": side,
            "type": BaseClient.ORDER_TYPE_MARKET,
            "quantity": abs(amount)
        }
        if trade_type == TradeType.COIN_M:
            func = self.client.futures_coin_create_order
        else:
            func = self.client.futures_create_order
        try:
            await func(**params)
        except Exception as e:
            logging.error(e)

    async def cancel_all_orders(self, trade_type, symbol):
        if trade_type == TradeType.COIN_M:
            func = self.client.futures_coin_cancel_all_open_orders
        elif trade_type == TradeType.USD_M:
            func = self.client.futures_cancel_all_open_orders
        else:
            func = self.client.cancel_all_orders

        await func(symbol=symbol)





import pytest
from binance.streams import BinanceSocketManager, AsyncClient, ReconnectingWebsocket
from binance.enums import FuturesType


@pytest.mark.asyncio
async def test_public_websocket(logger, proxy):
    client = AsyncClient()
    bm = BinanceSocketManager(client, proxy=proxy)

    ws: ReconnectingWebsocket = bm.index_price_socket("btcusd", True)
    await ws.__aenter__()
    # await ws.unsubscribe(["btcusdt@indexPrice@1s"])
    await ws.subscribe(["btcusd_210924@depth5", "btcusd_211231@depth5"])
    while True:
        res = await ws.recv()
        logger.info(res)

    async with bm.future_depth_socket("BTCUSD_210924", '5', FuturesType.COIN_M) as stream:
        while True:
            res = await stream.recv()
            logger.info(res)


@pytest.mark.asyncio
async def test_user_websocket(logger, proxy, binance_client: AsyncClient):
    bm = BinanceSocketManager(binance_client, proxy=proxy)
    socket = bm.user_socket()
    async with socket as stream:
        while True:
            res = await stream.recv()
            logger.info(res)


@pytest.mark.asyncio
async def test_coin_future_websocket(logger, proxy, binance_client: AsyncClient):
    bm = BinanceSocketManager(binance_client, proxy=proxy)
    socket = bm.coin_futures_socket()
    async with socket as stream:
        while True:
            res = await stream.recv()
            logger.info(res)



from logging import Logger

import pytest
from common.log import create_logger
from common.huobi.client import AsyncClient
from common.huobi.service import HuobiService
from common.okex.client import AsyncClient as OkexAsyncClient
from common.okex.stream import ReconnectingWebSocket
from binance.client import AsyncClient as BinanceAsyncClient
from .config import Huobi, Okex, Binance


@pytest.fixture(scope="function")
async def hb_client() -> AsyncClient:
    client = AsyncClient.create(Huobi.API_KEY, Huobi.API_SECRET)
    yield client
    await client.close_connection()


@pytest.fixture(scope="session")
def logger() -> Logger:
    return create_logger("pytest")


@pytest.fixture(scope="function")
async def huobi_service(hb_client) -> HuobiService:
    return HuobiService(hb_client)


@pytest.fixture()
async def okex_client() -> OkexAsyncClient:
    client = OkexAsyncClient.create(Okex.API_KEY, Okex.API_SECRET, Okex.API_PASSPHRASE, True)
    yield client
    await client.close_connection()


@pytest.fixture(scope="session")
def proxy() -> str:
    return "http://127.0.0.1:51838"


@pytest.fixture()
async def okex_pub_ws(proxy):
    return ReconnectingWebSocket(
        proxy=proxy,
        api_key=Okex.API_KEY,
        api_secret=Okex.API_SECRET,
        api_passphrase=Okex.API_PASSPHRASE,
        simulated=True
    )


@pytest.fixture()
async def okex_pri_ws(proxy):
    return ReconnectingWebSocket(
        public=False,
        proxy=proxy,
        api_key=Okex.API_KEY,
        api_secret=Okex.API_SECRET,
        api_passphrase=Okex.API_PASSPHRASE,
        simulated=True
    )


@pytest.fixture()
async def binance_client() -> BinanceAsyncClient:
    client = BinanceAsyncClient.create(Binance.API_KEY, Binance.API_SECRET)
    yield client
    await client.close_connection()



import pytest
from common.huobi.client import AsyncClient
from common.models import Result


def asset_ok(resp: Result):
    return resp.status == 'ok'


@pytest.mark.asyncio
async def test_kline(hb_client: AsyncClient, logger):
    resp: Result = await hb_client.spot_kline("btcusdt", hb_client.KLINE_INTERVAL_1MINUTE)
    assert asset_ok(resp)


@pytest.mark.asyncio
async def test_account_balance(hb_client: AsyncClient, logger):
    resp: Result = await hb_client.spot_account_info()
    assert asset_ok(resp)
    for item in resp.data:
        balance_info = await hb_client.spot_account_balance(item['id'])
        logger.info(balance_info.data)


@pytest.mark.asyncio
async def test_asset_valuation(hb_client: AsyncClient, logger):
    resp = await hb_client.spot_asset_valuation(hb_client.SPOT, "USD")
    logger.info(resp.data)




from binance import AsyncClient
from binance.client import Client


async def async_check_system_status(client: AsyncClient) -> bool:
    rst = await client.get_system_status()
    return rst['status'] == 0


async def async_check_account_status(client: AsyncClient) -> bool:
    rst = await client.get_account_status()
    return rst['data'] == "Normal"


def check_system_status(client: Client) -> bool:
    rst = client.get_system_status()
    return rst['status'] == 0


def check_account_status(client: Client) -> bool:
    rst = client.get_account_status()
    return rst['data'] == "Normal"


def check_api_trading_status(client: Client) -> bool:
    rst = client.get_account_api_trading_status()
    return rst['data']['isLocked'] is False


def check_can_trade(client: Client) -> bool:
    rst = client.get_account()
    return rst['canTrade'] is True



from constants import InstType


def parse_symbol(symbol: str) -> (InstType, str):
    symbol = symbol.upper()
    s = symbol.split('-')[0]

    if symbol.endswith("SWAP"):
        return InstType.SWAP, s
    if symbol.split('-')[-1].isdigit():
        return InstType.FUTURES, s
    return InstType.SPOT, s



import logging
from typing import Optional, Dict
from abc import ABC, abstractmethod
from datetime import datetime

import aiohttp

from common.huobi import constants
from .exceptions import HuobiAPIException
from common.models import Result
from .tools import generate_signature
from common.dict_util import package_data, remove_key


class BaseClient(ABC):
    # API URL
    API_URL = "https://api.huobi.pro"
    API_AWS_URL = "https://api-aws.huobi.pro"

    # 合约
    API_FUTURE_URL= "https://api.hbdm.com"

    # version
    SPOT_API_VERSION = 'v1'
    SPOT_API_VERSION2 = 'v2'

    FUTURE_API_VERSION = "v1"

    COIN_MARGINED_API_VERSION = "v1"

    USD_MARGINED_API_VERSION = "v1"

    # ACCOUNT_TYPE
    SPOT = "spot"   # 现货
    OTC = "otc"
    MARGIN = "margin"   # 逐仓杠杆账户，该账户类型以subType区分具体币种对账户
    SUPER_MARGIN = "super-margin"   # 全仓杠杆账户
    POINT = "point"     # 点卡
    MINEPOOL = "minepool"   # 矿池账户
    ETF = "etf"     # ETF账户

    KLINE_INTERVAL_1MINUTE = '1min'
    KLINE_INTERVAL_5MINUTE = '5min'
    KLINE_INTERVAL_15MINUTE = '15min'
    KLINE_INTERVAL_30MINUTE = '30min'
    KLINE_INTERVAL_60MINUTE = '60min'
    KLINE_INTERVAL_4HOUR = '4hour'
    KLINE_INTERVAL_1DAY = '1day'
    KLINE_INTERVAL_1WEEK = '1week'
    KLINE_INTERVAL_1MONTH = '1mon'
    KLINE_INTERVAL_1YEAR = '1year'

    DEPTH_TYPE_0 = "step0"
    DEPTH_TYPE_1 = "step1"
    DEPTH_TYPE_2 = "step2"
    DEPTH_TYPE_3 = "step3"
    DEPTH_TYPE_4 = "step4"
    DEPTH_TYPE_5 = "step5"

    TRANSFER_FUTURES_TO_PRO = "futures-to-pro"
    TRANSFER_PRO_TO_FUTURES = "pro-to-futures"

    def __init__(self, api_key: Optional[str] = None, api_secret: Optional[str] = None):
        self.API_KEY = api_key
        self.API_SECRET = api_secret
        self.session = self._init_session()

    @abstractmethod
    def _init_session(self):
        raise NotImplementedError

    @classmethod
    @abstractmethod
    def create(cls, api_key: Optional[str] = None, api_secret: Optional[str] = None):
        raise NotImplementedError

    def _create_spot_api_uri(self, path: str, version: str = SPOT_API_VERSION) -> str:
        return f"{self.API_URL}/{version}/{path}"

    def _create_spot_basic_api_uri(self, path: str, version: str = SPOT_API_VERSION) -> str:
        return f"{self.API_URL}/{version}/common/{path}"

    def _create_spot_market_api_uri(self, path: str) -> str:
        return f"{self.API_URL}/market/{path}"

    def _create_spot_account_api_uri(self, path: str, version: str = SPOT_API_VERSION) -> str:
        return f"{self.API_URL}/{version}/account/{path}"

    def _create_spot_sub_account_api_uri(self, path: str, version: str = SPOT_API_VERSION) -> str:
        return f"{self.API_URL}/{version}/subuser/{path}"

    def _create_spot_order_api_uri(self, path: str, version: str = SPOT_API_VERSION) -> str:
        return f"{self.API_URL}/{version}/order/{path}"

    def _create_spot_margin_api_uri(self, path: str, version: str = SPOT_API_VERSION) -> str:
        return f"{self.API_URL}/{version}/margin/{path}"

    def _create_spot_cross_margin_api_uri(self, path: str, version: str = SPOT_API_VERSION) -> str:
        return f"{self.API_URL}/{version}/cross-margin/{path}"

    # 交割合约
    def _create_future_api_uri(self, path: str, version: str = FUTURE_API_VERSION) -> str:
        return f"{self.API_FUTURE_URL}/api/{version}/{path}"

    # 币本位合约
    def _create_coin_margined_api_uri(self, path: str, version: str = COIN_MARGINED_API_VERSION) -> str:
        return f"{self.API_FUTURE_URL}/swap-api/{version}/{path}"

    # U本位合约
    def _create_usd_margined_api_uri(self, path: str, version: str = USD_MARGINED_API_VERSION) -> str:
        return f"{self.API_FUTURE_URL}/linear-swap-api/{version}/{path}"

    @staticmethod
    def _get_headers() -> Dict:
        return {'Content-type': 'application/x-www-form-urlencoded', 'Accept': 'application/json'}

    @staticmethod
    def _post_headers() -> Dict:
        return {'Accept': 'application/json', 'Content-type': 'application/json'}


class AsyncClient(BaseClient):
    def _init_session(self):
        return aiohttp.ClientSession(trust_env=True)

    async def close_connection(self):
        if self.session:
            await self.session.close()

    @classmethod
    def create(cls, api_key: Optional[str] = None, api_secret: Optional[str] = None) -> "AsyncClient":
        client = cls(api_key, api_secret)
        return client

    async def _request(self, method: str, uri: str, params=None, data=None, signed=False) -> Result:
        kwargs = dict()
        kwargs['timeout'] = 10
        kwargs['json'] = data

        if signed:
            timestamp = datetime.utcnow().strftime("%Y-%m-%dT%H:%M:%S")
            params = params if params else {}
            params.update({
                "AccessKeyId": self.API_KEY,
                "SignatureMethod": "HmacSHA256",
                "SignatureVersion": "2",
                "Timestamp": timestamp
            })
            params["Signature"] = generate_signature(self.API_SECRET, method, uri, params)

        if params:
            kwargs['params'] = params

        if method == "get":
            headers = self._get_headers()
        else:
            headers = self._post_headers()
        kwargs['headers'] = headers
        logging.debug(f"uri: {uri}, kwargs: {kwargs}")
        async with getattr(self.session, method)(uri, **kwargs) as response:
            return await self._handle_response(response)

    @staticmethod
    async def _handle_response(response: aiohttp.ClientResponse) -> Result:
        """Internal helper for handling API responses from the Binance server.
        Raises the appropriate exceptions when necessary; otherwise, returns the
        response.
        """
        if not str(response.status).startswith('2'):
            raise HuobiAPIException(response, await response.text())
        try:
            result = await response.json()
            logging.debug(f"handler response: {result}")
            # 统一v1/v2返回值格式
            if "code" in result:
                if result['code'] != 200:
                    raise HuobiAPIException(response, result['message'])
                else:
                    return Result(
                        result['code'], 'ok',
                        result.get('message'), result.get("data", result.get("tick"))
                    )
            else:
                if result['status'] != 'ok':
                    error_code = result.get('err-code', result.get('err_code'))
                    raise HuobiAPIException(
                        response,
                        f"error code: {error_code}, "
                        f"error msg: {result.get('err-msg', result.get('err_msg'))}",
                        error_code
                    )
                else:
                    return Result(200, result['status'], "", result.get("data", result.get("tick")))
        except ValueError:
            raise HuobiAPIException(response, f'Invalid Response: {await response.text()}')

    async def _get(self, path, params=None, signed=False):
        return await self._request('get', path, params, signed=signed)

    async def _post(self, path, data=None, signed=True):
        return await self._request('post', path, data=data, signed=signed)

    async def _request_spot_basic_api(self, method="get", path=None, version=BaseClient.SPOT_API_VERSION, params=None):
        uri = self._create_spot_basic_api_uri(path, version)
        return await self._request(method, uri, params)

    async def _request_spot_market_api(self, method="get", path=None, params=None):
        uri = self._create_spot_market_api_uri(path)
        return await self._request(method, uri, params)

    async def _request_spot_account_api(self, method="get", path=None, version=BaseClient.SPOT_API_VERSION,
                                        params=None, data=None):
        uri = self._create_spot_account_api_uri(path, version)
        return await self._request(method, uri, params, data, True)

    async def _request_spot_order_api(self, method="get", path=None, version=BaseClient.SPOT_API_VERSION,
                                      params=None, data=None):
        uri = self._create_spot_order_api_uri(path, version)
        return await self._request(method, uri, params, data, True)

    async def _request_future_private_api(self, method="get", path=None, version=BaseClient.SPOT_API_VERSION,
                                        params=None, data=None):
        uri = self._create_future_api_uri(path, version)
        return await self._request(method, uri, params, data, True)

    async def _request_coin_margined_private_api(self, method="get", path=None,
                                                 version=BaseClient.COIN_MARGINED_API_VERSION, params=None, data=None):
        uri = self._create_coin_margined_api_uri(path, version)
        return await self._request(method, uri, params, data, True)

    async def _request_usd_margined_private_api(
            self, method="get", path=None, version=BaseClient.USD_MARGINED_API_VERSION, params=None, data=None
    ):
        uri = self._create_usd_margined_api_uri(path, version)
        return await self._request(method, uri, params, data, True)

    async def spot_system_status(self):
        """
        获取当前系统状态
        https://huobiapi.github.io/docs/spot/v1/cn/#cd63bde415
        """
        return await self._request("get", "https://status.huobigroup.com/api/v2/summary.json")

    async def spot_market_status(self):
        """
        获取当前市场状态
        https://huobiapi.github.io/docs/spot/v1/cn/#f80d403388
        """
        uri = self._create_spot_api_uri("market-status", version=self.SPOT_API_VERSION2)
        return await self._request("get", uri)

    async def spot_symbols(self):
        """
        获取所有交易对
        https://huobiapi.github.io/docs/spot/v1/cn/#0e505d18dc
        """
        return await self._request_spot_basic_api(path="symbols")

    async def spot_currencys(self):
        """
        获取所有币种
        https://huobiapi.github.io/docs/spot/v1/cn/#7393cc8596
        """
        return await self._request_spot_basic_api(path="currencys")

    async def spot_timestamp(self):
        return await self._request_spot_basic_api(path="timestamp")

    async def spot_kline(self, symbol: str, period, **params):
        params['symbol'] = symbol
        params['period'] = period
        return await self._request_spot_market_api(path="history/kline", params=params)

    async def spot_merged_ticker(self, symbol: str):
        """
        聚合行情（Ticker）
        此接口获取ticker信息同时提供最近24小时的交易聚合信息。
        """
        return await self._request_spot_market_api(path="detail/merged", params={"symbol": symbol})

    async def spot_tickers(self):
        """
        所有交易对的最新 Tickers
        获得所有交易对的 tickers。 shell curl "https://api.huobi.pro/market/tickers"
        此接口返回所有交易对的 ticker，因此数据量较大。
        """
        return await self._request_spot_market_api(path="tickers")

    async def spot_depth(self, symbol: str, depth_type: Optional[str] = BaseClient.DEPTH_TYPE_0, depth: Optional[int] = 20):
        """
        市场深度数据
        depth 范围 5，10，20
        当type值为‘step0’时，‘depth’的默认值为150而非20
        参数type的各值说明

        取值	说明
        step0	无聚合
        step1	聚合度为报价精度*10
        step2	聚合度为报价精度*100
        step3	聚合度为报价精度*1000
        step4	聚合度为报价精度*10000
        step5	聚合度为报价精度*100000
        """
        params = {"symbol": symbol, "depth": depth, "type": depth_type}
        return await self._request_spot_market_api(path="depth", params=params)

    async def spot_trade(self, symbol: str):
        """
        返回指定交易对最新的一个交易记录
        """
        return await self._request_spot_market_api(path="trade", params={'symbol': symbol})

    async def spot_trade_history(self, symbol: str, size: Optional[int] = 1):
        """
        返回指定交易对近期的所有交易记录
        size 最大值 2000
        """
        assert size <= 2000
        return await self._request_spot_market_api(path="history/trade", params={"symbol": symbol, "size": size})

    async def spot_market_detail(self, symbol: str):
        """
        最近24小时的行情数据汇总
        此接口返回的成交量、成交金额为24小时滚动数据（平移窗口大小24小时），有可能会出现后一个窗口内的累计成交量、累计成交额小于前一窗口的情况。
        """
        return await self._request_spot_market_api(path="detail", params={"symbol": symbol})

    async def spot_etf(self, symbol: str):
        """
        杠杆ETP的最新净值
        """
        return await self._request_spot_market_api(path="etf", params={'symbol': symbol})

    async def spot_account_info(self):
        """
        查询当前用户的所有账户 ID account-id 及其相关信息
        """
        return await self._request_spot_account_api(path="accounts")

    async def spot_account_balance(self, account_id: str):
        """
        查询指定账户的余额，支持以下账户：

        spot：现货账户， margin：逐仓杠杆账户，otc：OTC 账户，point：点卡账户，super-margin：全仓杠杆账户,
        investment: C2C杠杆借出账户, borrow: C2C杠杆借入账户
        """
        return await self._request_spot_account_api(path=f"accounts/{account_id}/balance")

    async def spot_asset_valuation(self, account_type: str, currency: Optional[str] = "BTC"):
        """
        按照BTC或法币计价单位，获取指定账户的总资产估值
        """
        return await self._request_spot_account_api(
            path="asset-valuation", version=BaseClient.SPOT_API_VERSION2,
            params={'accountType': account_type, "valuationCurrency": currency}
        )

    async def spot_cancel_order(self, order_id):
        uri = self._create_spot_order_api_uri(f"orders/{order_id}/submitcancel")
        return await self._request_spot_order_api("post", uri)

    async def future_transfer(self, currency: str, amount, transfer_type: str):
        """
        币币现货账户与交割合约账户划转
        从现货现货账户转至交割合约账户，类型为pro-to-futures
        从交割合约账户转至现货账户，类型为futures-to-pro
        """
        uri = self._create_spot_api_uri("futures/transfer")
        data = {
            "currency": currency,
            "amount": amount,
            "type": transfer_type
        }
        return await self._request("post", uri, data=data, signed=True)

    async def coin_margined_transfer(self, _from, _to, currency, amount):
        """
        现货-币本位永续合约账户之间进行资金的划转

        此接口用户币币现货账户与币本位永续合约账户之间的资金划转。
        该接口的访问频次的限制为1秒/1次。
        现货与币本位永续合约划转接口，所有划转的币的精度是8位小数

        from	true	string	来源业务线账户，取值：spot(币币)、swap(币本位永续)	e.g. spot
        to	true	string	目标业务线账户，取值：spot(币币)、swap(币本位永续)	e.g. swap
        currency	true	string	币种,支持大小写	e.g. btc
        amount	true	Decimal	划转金额
        """
        uri = self._create_spot_api_uri("account/transfer", version=BaseClient.SPOT_API_VERSION2)
        data = {
            "from": _from,
            "to": _to,
            "currency": currency,
            "amount": amount
        }
        return await self._request("post", uri, data=data, signed=True)


    ##############################
    # 交割合约
    ##############################

    #### 市场行情 ####

    async def future_contract_info(self, symbol=None, contract_type=None, contract_code=None):
        """
        symbol	string	false	支持大小写，"BTC","ETH"...
        contract_type	string	false	合约类型: （this_week:当周 next_week:下周 quarter:当季 next_quarter:次季）
        contract_code	string	false	BTC180914
        """
        uri = self._create_future_api_uri("contract_contract_info")
        params = {}
        if symbol:
            params['symbol'] = symbol
        if contract_type:
            params['contract_type'] = contract_type
        if contract_code:
            params['contract_code'] = contract_code
        return await self._request("get", uri, params=params)

    async def future_contract_index(self, symbol=None):
        """
        合约指数
        :param symbol:
        :return:
        """
        uri = self._create_future_api_uri("contract_index")
        params = {}
        if symbol:
            params['symbol'] = symbol
        return await self._request("get", uri, params=params)

    async def future_contract_price_limit(self, symbol=None, contract_type=None, contract_code=None):
        """
        获取合约最高/低限价
        symbol	string	false	支持大小写，"BTC","ETH"...
        contract_type	string	false	合约类型: （this_week:当周 next_week:下周 quarter:当季 next_quarter:次季）
        contract_code	string	false	BTC180914
        """
        uri = self._create_future_api_uri("contract_price_limit")
        params = {}
        if symbol:
            params['symbol'] = symbol
        if contract_type:
            params['contract_type'] = contract_type
        if contract_code:
            params['contract_code'] = contract_code
        return await self._request("get", uri, params=params)

    async def future_contract_open_interest(self, symbol=None, contract_type=None, contract_code=None):
        """
        当前合约的总持仓量
        symbol	string	false	支持大小写，"BTC","ETH"...
        contract_type	string	false	合约类型: （this_week:当周 next_week:下周 quarter:当季 next_quarter:次季）
        contract_code	string	false	BTC180914
        """
        uri = self._create_future_api_uri("contract_open_interest")
        params = {}
        if symbol:
            params['symbol'] = symbol
        if contract_type:
            params['contract_type'] = contract_type
        if contract_code:
            params['contract_code'] = contract_code
        return await self._request("get", uri, params=params)

    async def future_contract_delivery_price(self, symbol=None):
        """
        预估交割价
        :param symbol:
        :return:
        """
        uri = self._create_future_api_uri("contract_delivery_price")
        params = {}
        if symbol:
            params['symbol'] = symbol
        return await self._request("get", uri, params=params)

    async def future_contract_estimated_settlement_price(self, symbol=None):
        """
        预估结算价
        :param symbol:
        :return:
        """
        params = package_data(remove_key(locals()))
        uri = self._create_future_api_uri("contract_estimated_settlement_price")
        return await self._request("get", uri, params=params)

    async def future_api_state(self, symbol=None):
        """
        查询系统状态
        :param symbol:
        :return:
        """
        params = package_data(remove_key(locals()))
        uri = self._create_future_api_uri("contract_api_state")
        return await self._request("get", uri, params=params)

    async def future_market_depth(self, symbol, depth_type = constants.WS_DEPTH_5):
        """
        获取行情深度
        :param symbol:
        :param depth_type:
        :return:
        """
        params = package_data(remove_key(locals()), replace_keys={"depth_type": "type"})
        uri = f"{self.API_FUTURE_URL}/market/depth"
        return await self._request("get", uri, params=params)

    async def future_market_bbo(self, symbol=None):
        """
        获取最优挂单
        symbol	false	string	合约标识，不填返回全部合约的最优挂单信息
        如"BTC_CW"表示BTC当周合约，"BTC_NW"表示BTC次周合约，"BTC_CQ"表示BTC当季合约 ，"BTC_NQ"表示BTC次季度合约。
        支持使用合约code来查询数据， 例如："BTC200918"(当周)，"BTC200925"(次周)，"BTC201225"(季度)，"BTC210326"(次季度)。
        """
        params = package_data(remove_key(locals()))
        uri = f"{self.API_FUTURE_URL}/market/bbo"
        return await self._request("get", uri, params=params)

    async def future_kline(self, symbol, period, size=None, start=None, end=None):
        """
        获取K线数据
        symbol	false	string	合约标识，不填返回全部合约的最优挂单信息
        如"BTC_CW"表示BTC当周合约，"BTC_NW"表示BTC次周合约，"BTC_CQ"表示BTC当季合约 ，"BTC_NQ"表示BTC次季度合约。
        支持使用合约code来查询数据， 例如："BTC200918"(当周)，"BTC200925"(次周)，"BTC201225"(季度)，"BTC210326"(次季度)。
        period	true	string	K线类型		1min, 5min, 15min, 30min, 60min,4hour,1day, 1mon
        size	false	int	获取数量	150	[1,2000]
        from	false	long	开始时间戳 10位 单位S
        to	false	long	结束时间戳 10位 单位S

        Note
        1、size字段或者from、to字段至少要填写一个。
        2、如果size、from、to 均不填写，则返回错误。
        3、如果填写from，也要填写to。最多可获取连续两年的数据。
        4、如果size、from、to 均填写，会忽略from、to参数。
        支持查询已下市四周内的合约的K线数据（即到期日在过去最近四周的合约），可输入已下市四周内的合约的合约代码进行查询K线数据。
        """
        params = package_data(remove_key(locals()), {"start": "from", "end": "to"})
        uri = f"{self.API_FUTURE_URL}/market/history/kline"
        return await self._request("get", uri, params=params)

    async def future_mark_price_kline(self, symbol, period, size):
        """
        获取标记价格的K线数据
        symbol	false	string	合约标识，不填返回全部合约的最优挂单信息
        如"BTC_CW"表示BTC当周合约，"BTC_NW"表示BTC次周合约，"BTC_CQ"表示BTC当季合约 ，"BTC_NQ"表示BTC次季度合约。
        支持使用合约code来查询数据， 例如："BTC200918"(当周)，"BTC200925"(次周)，"BTC201225"(季度)，"BTC210326"(次季度)。
        period	true	string	K线类型		1min, 5min, 15min, 30min, 60min,4hour,1day, 1mon
        size	false	int	获取数量	150	[1,2000]

        Note
        1、最多一次2000条数据
        """
        params = package_data(remove_key(locals()))
        uri = f"{self.API_FUTURE_URL}/index/market/history/mark_price_kline"
        return await self._request("get", uri, params=params)

    async def future_merged(self, symbol):
        """
        获取聚合行情
        symbol	false	string	合约标识，不填返回全部合约的最优挂单信息
        如"BTC_CW"表示BTC当周合约，"BTC_NW"表示BTC次周合约，"BTC_CQ"表示BTC当季合约 ，"BTC_NQ"表示BTC次季度合约。
        支持使用合约code来查询数据， 例如："BTC200918"(当周)，"BTC200925"(次周)，"BTC201225"(季度)，"BTC210326"(次季度)。
        """
        params = package_data(remove_key(locals()))
        uri = f"{self.API_FUTURE_URL}/market/detail/merged"
        return await self._request("get", uri, params=params)

    async def future_batch_merged(self, symbol=None):
        """
        批量获取聚合行情
        symbol	false	string	合约标识，不填返回全部合约的最优挂单信息
        如"BTC_CW"表示BTC当周合约，"BTC_NW"表示BTC次周合约，"BTC_CQ"表示BTC当季合约 ，"BTC_NQ"表示BTC次季度合约。
        支持使用合约code来查询数据， 例如："BTC200918"(当周)，"BTC200925"(次周)，"BTC201225"(季度)，"BTC210326"(次季度)。

        该接口更新频率为50ms
        """
        params = package_data(remove_key(locals()))
        uri = f"{self.API_FUTURE_URL}/market/detail/batch_merged"
        return await self._request("get", uri, params=params)

    async def future_trade_history(self, symbol=None):
        """
        获取市场最近成交记录
        symbol	false	string	合约标识，不填返回全部合约的最优挂单信息
        如"BTC_CW"表示BTC当周合约，"BTC_NW"表示BTC次周合约，"BTC_CQ"表示BTC当季合约 ，"BTC_NQ"表示BTC次季度合约。
        支持使用合约code来查询数据， 例如："BTC200918"(当周)，"BTC200925"(次周)，"BTC201225"(季度)，"BTC210326"(次季度)。
        """
        params = package_data(remove_key(locals()))
        uri = f"{self.API_FUTURE_URL}/market/trade"
        return await self._request("get", uri, params=params)

    async def future_batch_trade_history(self, symbol, size):
        """
        批量获取市场最近成交记录
        symbol	true	string	合约标识，不填返回全部合约的最优挂单信息
        如"BTC_CW"表示BTC当周合约，"BTC_NW"表示BTC次周合约，"BTC_CQ"表示BTC当季合约 ，"BTC_NQ"表示BTC次季度合约。
        支持使用合约code来查询数据， 例如："BTC200918"(当周)，"BTC200925"(次周)，"BTC201225"(季度)，"BTC210326"(次季度)。
        size	true	int	获取数量	150	[1,2000]
        """
        params = package_data(remove_key(locals()))
        uri = f"{self.API_FUTURE_URL}/market/history/trade"
        return await self._request("get", uri, params=params)

    async def future_liquidation_orders(self, symbol, trade_type, create_date, page_index=1, page_size=20):
        """
        获取强平订单
        symbol	true	string	品种代码		支持大小写，"BTC","ETH"...
        trade_type	true	int	交易类型		0:全部,5: 卖出强平,6: 买入强平
        create_date	true	int	日期		7，90（7天或者90天）
        page_index	false	int	页码,不填默认第1页	1
        page_size	false	int	不填默认20，不得多于50	20	[1-50]
        """
        params = package_data(remove_key(locals()))
        uri = self._create_future_api_uri("contract_liquidation_orders")
        return await self._request("get", uri, params=params)

    async def future_basis_data(self, symbol, period, basis_price_type=constants.BasisPriceType.OPEN, size=100):
        """
        获取基差数据
        symbol	true	string	合约名称		支持大小写，如"BTC_CW"表示BTC当周合约，"BTC_NW"表示BTC次周合约，"BTC_CQ"表示BTC当季合约, "BTC_NQ"表示次季度合约"
        period	true	string	周期		仅支持小写，1min,5min, 15min, 30min, 60min,4hour,1day,1mon
        basis_price_type	false	string	基差价格类型，表示在周期内计算基差使用的价格类型	不填，默认使用开盘价	仅支持小写，开盘价：open，收盘价：close，最高价：high，最低价：low，平均价=（最高价+最低价）/2：average
        size	true	int	基差获取数量		[1,2000]
        """
        params = package_data(remove_key(locals()))
        uri = f"{self.API_FUTURE_URL}/index/market/history/basis"
        return await self._request("get", uri, params=params)

    #### 合约资产 #####

    async def future_balance_valuation(self, valuation_asset="BTC"):
        """
        获取账户总资产估值
        valuation_asset	false	string	资产估值币种，即按该币种为单位进行估值，不填默认"BTC"	"BTC","USD","CNY","EUR","GBP","VND","HKD","TWD","MYR","SGD","KRW","RUB","TRY"
        """
        data = {"valuation_asset": valuation_asset}
        return await self._request_future_private_api("post", "contract_balance_valuation", data=data)

    async def future_account_info(self, symbol):
        """
        获取账户信息
        symbol	false	string	品种代码		支持大小写,"BTC","ETH"...如果缺省，默认返回所有品种
        """
        data = package_data(remove_key(locals()))
        rst = await self._request_future_private_api("post", "contract_account_info", data=data)
        if rst.status == 'ok' and rst.data and symbol:
            new_data = None
            for item in rst.data:
                if item['symbol'].upper() == symbol.upper():
                    new_data = item
                    break
            return Result(rst.code, rst.status, rst.msg, new_data)
        return rst

    async def future_position_info(self, symbol):
        """
        获取持仓信息
        symbol	false	string	品种代码		支持大小写,"BTC","ETH"...如果缺省，默认返回所有品种
        """
        data = package_data(remove_key(locals()))
        return await self._request_future_private_api("post", "contract_position_info", data=data)

    async def future_fee(self, symbol=None):
        """
        查询用户当前的手续费费率
        symbol	false	string	品种代码		支持大小写,"BTC","ETH"...如果缺省，默认返回所有品种
        """
        data = package_data(remove_key(locals()))
        return await self._request_future_private_api("post", "contract_fee", data=data)

    async def future_position_limit(self, symbol=None):
        """
        用户持仓量限制的查询
        symbol	false	string	品种代码		支持大小写,"BTC","ETH"...如果缺省，默认返回所有品种
        """
        data = package_data(remove_key(locals()))
        return await self._request_future_private_api("post", "contract_position_limit", data=data)

    async def future_account_position_info(self, symbol):
        """
        查询用户账户和持仓信息
        symbol	true	string	品种代码		支持大小写,"BTC","ETH"...如果缺省，默认返回所有品种
        """
        params = {"symbol": symbol}
        return await self._request_future_private_api("post", "contract_account_position_info", params=params)

    #### 合约交易接口 ####
    async def future_create_order(
            self, direction, volume: int, offset, order_price_type, lever_rate=None, symbol=None, contract_type=None,
            contract_code=None, client_order_id=None, price=None, tp_trigger_price=None, tp_order_price=None,
            tp_order_price_type=None, sl_trigger_price=None, sl_order_price=None, sl_order_price_type=None):
        """
        开单
        symbol	string	false	支持大小写,"BTC","ETH"...
        contract_type	string	false	合约类型 ("this_week":当周 "next_week":下周 "quarter":当季 "next_quarter":次季)
        contract_code	string	false	BTC180914
        client_order_id	long	false	客户自己填写和维护，必须为数字,请注意必须小于等于9223372036854775807
        price	decimal	false	价格
        volume	long	true	委托数量(张)
        direction	string	true	"buy":买 "sell":卖
        offset	string	true	"open":开 "close":平
        lever_rate	int	true	杠杆倍数[“开仓”若有10倍多单，就不能再下20倍多单, "平仓"可以不填杠杆倍数;首次使用高倍杠杆(>20倍)，请使用主账号登录web端同意高倍杠杆协议后，才能使用接口下高倍杠杆(>20倍)]
        order_price_type	string	true	订单报价类型 "limit":限价，"opponent":对手价 ，"post_only":只做maker单,post only下单只受用户持仓数量限制,"optimal_5"：最优5档，"optimal_10"：最优10档，"optimal_20"：最优20档，"ioc":IOC订单，"fok"：FOK订单, "opponent_ioc"： 对手价-IOC下单，"optimal_5_ioc"：最优5档-IOC下单，"optimal_10_ioc"：最优10档-IOC下单，"optimal_20_ioc"：最优20档-IOC下单,"opponent_fok"： 对手价-FOK下单，"optimal_5_fok"：最优5档-FOK下单，"optimal_10_fok"：最优10档-FOK下单，"optimal_20_fok"：最优20档-FOK下单
        tp_trigger_price	decimal	false	止盈触发价格
        tp_order_price	decimal	false	止盈委托价格（最优N档委托类型时无需填写价格）
        tp_order_price_type	string	false	止盈委托类型 不填默认为limit; 限价：limit ，最优5档：optimal_5，最优10档：optimal_10，最优20档：optimal_20
        sl_trigger_price	decimal	false	止损触发价格
        sl_order_price	decimal	false	止损委托价格（最优N档委托类型时无需填写价格）
        sl_order_price_type	string	false	止损委托类型 不填默认为limit; 限价：limit ，最优5档：optimal_5，最优10档：optimal_10，最优20档：optimal_20

        备注
        如果contract_code填了值，那就按照contract_code去下单，如果contract_code没有填值，则按照symbol+contract_type去下单。

        对手价下单price价格参数不用传，对手价下单价格是买一和卖一价,optimal_5：最优5档、optimal_10：最优10档、optimal_20：最优20档下单price价格参数不用传，"limit":限价，"post_only":只做maker单 需要传价格，"fok"：全部成交或立即取消，"ioc":立即成交并取消剩余。

        Post only(也叫maker only订单，只下maker单)每个周期合约的开仓/平仓的下单数量限制为500000，同时也会受到用户持仓数量限制。

        若存在持仓，那么下单时杠杆倍数必须与持仓杠杆相同，否则下单失败。若需使用新杠杆下单，则必须先使用切换杠杆接口将持仓杠杆切换成功后再下单。

        只有开仓订单才支持设置止盈止损。

        止盈触发价格为设置止盈单必填字段，止损触发价格为设置止损单必填字段；若缺省触发价格字段则不会设置对应的止盈单或止损单。

        开平方向
        开多：买入开多(direction用buy、offset用open)

        平多：卖出平多(direction用sell、offset用close)

        开空：卖出开空(direction用sell、offset用open)

        平空：买入平空(direction用buy、offset用close)
        """
        data = package_data(remove_key(locals()))
        return await self._request_future_private_api("post", "contract_order", data=data)

    async def future_batch_create_order(self, orders_data: list):
        """
        批量开单
        :param orders_data: 参数同开单接口
        :return:
        """
        return await self._request_future_private_api("post", "contract_batchorder", data={"orders_data": orders_data})

    async def future_cancel_order(self, symbol, order_id=None, client_order_id=None):
        """
        撤销订单
        order_id	false (请看备注)	string	订单ID(多个订单ID中间以","分隔,一次最多允许撤消10个订单)
        client_order_id	false (请看备注)	string	客户订单ID(多个订单ID中间以","分隔,一次最多允许撤消10个订单)
        symbol	true	string	支持大小写,"BTC","ETH"...

        备注：
        order_id和client_order_id都可以用来撤单，至少要填写一个,同时只可以设置其中一种，如果设置了两种，默认以order_id来撤单。

        撤单接口返回结果只代表撤单命令发送成功，建议根据订单查询接口查询订单的状态来确定订单是否已真正撤销。

        client_order_id，24小时有效，超过24小时的订单根据client_order_id将查询不到
        """
        assert symbol is not None
        data = package_data(remove_key(locals()))
        return await self._request_future_private_api("post", "contract_cancel", data=data)

    async def future_cancel_all(self, symbol, contract_code=None, contract_type=None, direction=None, offset=None):
        """
        全部撤单
        symbol	true	string	品种代码，支持大小写，如"BTC","ETH"...
        contract_code	false	string	合约code
        contract_type	false	string	合约类型
        direction	false	string	买卖方向（不填默认全部） "buy":买 "sell":卖
        offset	false	string	开平方向（不填默认全部）"open":开 "close":平

        备注
        只传symbol，撤销该品种下所有周期的合约
        只要有contract_code，则撤销该code的合约
        只传symbol+contract_type， 则撤销二者拼接所成的合约订单
        direction与offset可只填其一，只填其一则按对应的条件去撤单。（如用户只传了direction=buy，则撤销所有买单，包括开仓和平仓）
        """
        data = package_data(remove_key(locals()))
        return await self._request_future_private_api("post", "contract_cancelall", data=data)

    async def future_switch_lever_rate(self, symbol, lever_rate: int):
        """
        切换杠杆
        备注
        只有在单个品种下只有持仓，且没有挂单的场景下，才可以切换该品种当前的倍数。
        接口限制请求次数为每3秒一次。
        """
        data = package_data(remove_key(locals()))
        return await self._request_future_private_api("post", "contract_switch_lever_rate", data=data)

    async def future_order_info(self, symbol, order_id=None, client_order_id=None):
        """
        订单信息
        参数名称	是否必须	类型	描述
        order_id	false（请看备注）	string	订单ID(多个订单ID中间以","分隔,一次最多允许查询50个订单)
        client_order_id	false（请看备注）	string	客户订单ID(多个订单ID中间以","分隔,一次最多允许查询50个订单)
        symbol	true	string	支持大小写，"BTC","ETH"...

        最多只能查询 4 小时内的撤单信息。
        order_id和client_order_id至少要填写一个。
        order_id和client_order_id都可以用来查询，同时只可以设置其中一种，如果设置了两种，默认以order_id来查询。每天结算或周五交割后，会把结束状态的订单（5部分成交已撤单 6全部成交 7已撤单）删除掉。
        client_order_id，24小时有效，超过24小时的订单根据client_order_id将查询不到。
        """
        data = package_data(remove_key(locals()))
        return await self._request_future_private_api("post", "contract_order_info", data=data)

    async def future_order_detail(self, symbol, order_id, create_at: int = None, order_type=None, page_index=None,
                                  page_size=None):
        """
        订单明细信息
        参数名称	是否必须	类型	描述
        symbol	true	string	支持大小写,"BTC","ETH"...
        order_id	true	long	订单id
        created_at	false	long	下单时间戳
        order_type	false	int	订单类型，1:报单 、 2:撤单 、 3:强平、4:交割
        page_index	false	int	第几页,不填第一页
        page_size	false	int	不填默认20,不得多于50

        获取订单明细接口查询撤单数据时，如果传“created_at”和“order_type”参数则能查询最近6小时数据，如果不传“created_at”和“order_type”参数只能查询到最近2小时数据。
        order_id返回是18位，nodejs和javascript默认解析18有问题，nodejs和javascript里面JSON.parse默认是int，超过18位的数字用json-bigint的包解析。
        created_at使用13位long类型时间戳（包含毫秒时间），如果输入准确的时间戳，查询性能将会提升。例如:"2019/10/18 10:26:22"转换为时间戳为：1571365582123。也可以直接从contract_order下单接口返回的ts中获取时间戳查询对应的订单。
        created_at禁止传0。
        """
        data = package_data(remove_key(locals()))
        return await self._request_future_private_api("post", "contract_order_detail", data=data)

    async def future_open_orders(self, symbol, page_index=1, page_size=20, sort_by="created_at", trade_type=0):
        """
        获取合约当前未成交委托

        参数名称	是否必须	类型	描述	默认值	取值范围
        symbol	true	string	品种代码		支持大小写, "BTC","ETH"...
        page_index	false	int	页码，不填默认第1页	1
        page_size	false	int	不填默认20，不得多于50	20	[1-50]
        sort_by	false	string	排序字段，不填默认按创建时间倒序	created_at	“created_at”(按照创建时间倒序)，“update_time”(按照更新时间倒序)
        trade_type	false	int	交易类型，不填默认查询全部	0	0:全部,1:买入 开多,2: 卖出开空,3: 买入平空,4: 卖出平多。
        """
        data = package_data(remove_key(locals()))
        return await self._request_future_private_api("post", "contract_openorders", data=data)

    async def future_contract_hisorders(self, symbol, trade_type, type, status, create_date, page_index=1, page_size=20,
                                        contract_code=None, order_type=None, sort_by="create_date"):
        """
        获取合约历史委托
        symbol	true	string	品种代码		支持大小写,"BTC","ETH"...
        trade_type	true	int	交易类型		0:全部,1:买入开多,2: 卖出开空,3: 买入平空,4: 卖出平多,5: 卖出强平,6: 买入强平,7:交割平多,8: 交割平空, 11:减仓平多，12:减仓平空
        type	true	int	类型		1:所有订单,2:结束状态的订单
        status	true	string	订单状态		可查询多个状态，"3,4,5" , 0:全部,3:未成交, 4: 部分成交,5: 部分成交已撤单,6: 全部成交,7:已撤单
        create_date	true	int	日期		可随意输入正整数, ，如果参数超过90则默认查询90天的数据
        page_index	false	int	页码，不填默认第1页	1
        page_size	false	int	每页条数，不填默认20,不得多于50	20	[1-50]
        contract_code	false	string	合约代码
        order_type	false	string	订单类型		1：限价单、3：对手价、4：闪电平仓、5：计划委托、6：post_only、7：最优5档、8：最优10档、9：最优20档、10：fok、11：ioc
        sort_by	false	string	排序字段（降序），不填默认按照create_date降序	create_date	"create_date"：按订单创建时间进行降序，"update_time"：按订单更新时间进行降序
        """
        data = package_data(remove_key(locals()))
        return await self._request_future_private_api("post", "contract_hisorders", data=data)

    async def future_contract_hisorders_exact(self, symbol, trade_type, type, status, contract_code=None,
                                              order_price_type=None, start_time=None, end_time=None, from_id=None,
                                              size=20, direct="prev"):
        """
        组合查询合约历史委托
        symbol 	true 	string	品种代码 	"BTC","ETH"...
        trade_type	true	int	交易类型	0:全部,1:买入开多,2: 卖出开空,3: 买入平空,4: 卖出平多,5: 卖出强平,6: 买入强平,7:交割平多,8: 交割平空, 11:减仓平多，12:减仓平空
        type	true	int	类型	1:所有订单,2:结束状态的订单
        status	true	string	订单状态	可查询多个状态，"3,4,5" , 0:全部,3:未成交, 4: 部分成交,5: 部分成交已撤单,6: 全部成交,7:已撤单
        contract_code	false	string	合约代码
        order_price_type	false	string	订单报价类型	订单报价类型 "limit":限价，"opponent":对手价 ，"post_only":只做maker单,post only下单只受用户持仓数量限制,"optimal_5"：最优5档，"optimal_10"：最优10档，"optimal_20"：最优20档，"ioc":IOC订单，"fok"：FOK订单, "opponent_ioc"： 对手价-IOC下单，"optimal_5_ioc"：最优5档-IOC下单，"optimal_10_ioc"：最优10档-IOC下单，"optimal_20_ioc"：最优20档-IOC下单,"opponent_fok"： 对手价-FOK下单，"optimal_5_fok"：最优5档-FOK下单，"optimal_10_fok"：最优10档-FOK下单，"optimal_20_fok"：最优20档-FOK下单
        start_time	false	long	起始时间（时间戳，单位毫秒）	详见备注
        end_time	false	long	结束时间（时间戳，单位毫秒）	详见备注
        from_id	false	long	查询起始id（取返回数据的query_id字段）
        size	false	int	数据条数，默认取20，最大50	[1-50]
        direct	false	string	查询方向	prev 向前；next 向后；默认值取prev
        """
        data = package_data(remove_key(locals()))
        return await self._request_future_private_api("post", "contract_hisorders_exact", data=data)

    async def future_match_result(self, symbol, trade_type, create_date, contract_code=None, page_index=1, page_size=20):
        """
        获取历史成交记录

        参数名称	是否必须	类型	描述	默认值	取值范围
        symbol	true	string	品种代码		支持大小写,"BTC","ETH"...
        trade_type	true	int	交易类型		0:全部,1:买入开多,2: 卖出开空,3: 买入平空,4: 卖出平多,5: 卖出强平,6: 买入强平
        create_date	true	int	日期		可随意输入正整数，如果参数超过90则默认查询90天的数据
        contract_code	false	string	合约code
        page_index	false	int	页码，不填默认第1页	1
        page_size	false	int	不填默认20，不得多于50	20	[1-50]
        """
        data = package_data(remove_key(locals()))
        return await self._request_future_private_api("post", "contract_matchresults", data=data)

    async def future_match_result_exact(self, symbol, trade_type, contract_code=None, start_time=None, end_time=None,
                                        from_id=None, size=20, direct="prev"):
        """
        组合查询历史成交记录接口
        symbol	true	string	品种代码	"BTC","ETH"...
        trade_type	true	int	交易类型	0:全部,1:买入开多,2: 卖出开空,3: 买入平空,4: 卖出平多,5: 卖出强平,6: 买入强平
        contract_code	false	string	合约code
        start_time	false	long	起始时间（时间戳，单位毫秒）	详见备注
        end_time	false	long	结束时间（时间戳，单位毫秒）	详见备注
        from_id	false	long	查询起始id（取返回数据的query_id字段）
        size	false	int	数据条数,默认取20，最大50	[1-50]
        direct	false	string	查询方向	prev 向前；next 向后；默认值取prev
        """
        data = package_data(remove_key(locals()))
        return await self._request_future_private_api("post", "contract_matchresults_exact", data=data)

    async def future_lightning_close_position(self, volume, direction, symbol=None, contract_type=None,
                                              contract_code=None, client_order_id=None,
                                              order_price_type=constants.OrderPriceType.LIGHTNING):
        """
        闪电平仓
        """
        data = package_data(remove_key(locals()))
        return await self._request_future_private_api("post", "lightning_close_position", data=data)

    #### 币本位永续合约行情接口 #####
    async def coin_margined_contract_info(self, contract_code=None):
        """
        合约信息
        contract_code	string	false	合约代码, 大小写均支持，"BTC-USD",不填查询所有合约
        """
        params = package_data(remove_key(locals()))
        uri = self._create_coin_margined_api_uri("swap_contract_info")
        return await self._request("get", uri, params=params)

    async def coin_margined_contract_index(self, contract_code=None):
        """
        合约指数信息
        contract_code	string	false	合约代码, 大小写均支持，"BTC-USD",不填查询所有合约
        """
        params = package_data(remove_key(locals()))
        uri = self._create_coin_margined_api_uri("swap_index")
        return await self._request("get", uri, params=params)

    async def coin_margined_open_interest(self, contract_code=None):
        """
        合约总持仓量
        contract_code	string	false	合约代码, 大小写均支持，"BTC-USD",不填查询所有合约
        """
        params = package_data(remove_key(locals()))
        uri = self._create_coin_margined_api_uri("swap_open_interest")
        return await self._request("get", uri, params=params)

    async def coin_margined_market_depth(self, contract_code, depth_type = constants.WS_DEPTH_5):
        """
        获取行情深度
        contract_code	string	true	合约代码，支持大小写, "BTC-USD" ...
        type	string	true	(150档数据) step0, step1, step2, step3, step4, step5, step14, step15, step16, step17（合并深度1-5,14-17）；step0时，不合并深度, (20档数据) step6, step7, step8, step9, step10, step11, step12, step13, step18, step19（合并深度7-13,18-19）；step6时，不合并深度
        """
        params = package_data(remove_key(locals()), replace_keys={"depth_type": "type"})
        uri = f"{self.API_FUTURE_URL}/swap-ex/market/depth"
        return await self._request("get", uri, params=params)

    async def coin_margined_market_bbo(self, contract_code=None):
        """
        获取最优挂单
        contract_code	false	string	合约代码，不填返回全部合约的市场最优挂单信息	"BTC-USD" ...
        """
        params = package_data(remove_key(locals()))
        uri = f"{self.API_FUTURE_URL}/swap-ex/market/bbo"
        return await self._request("get", uri, params=params)

    async def coin_margined_merged(self, contract_code):
        """
        获取聚合行情
        contract_code	true	string	合约代码	仅支持大写， "BTC-USD" ...
        """
        params = package_data(remove_key(locals()))
        uri = f"{self.API_FUTURE_URL}/swap-ex/market/detail/merged"
        return await self._request("get", uri, params=params)

    async def coin_margined_api_state(self, contract_code=None):
        """
        查询系统状态
        :param contract_code:
        :return:
        """
        params = package_data(remove_key(locals()))
        uri = self._create_coin_margined_api_uri("swap_api_state")
        return await self._request("get", uri, params=params)

    async def coin_margined_funding_rate(self, contract_code):
        """
        查询合约资金费率
        :param contract_code:
        :return:
        """
        params = package_data(remove_key(locals()))
        uri = self._create_coin_margined_api_uri("swap_funding_rate")
        return await self._request("get", uri, params=params)

    async def coin_margined_batch_funding_rate(self, contract_code=None):
        """
        批量查询合约资金费率
        :param contract_code:
        :return:
        """
        params = package_data(remove_key(locals()))
        uri = self._create_coin_margined_api_uri("swap_batch_funding_rate")
        return await self._request("get", uri, params=params)

    async def coin_margined_liquidation_orders(self, contract_code, trade_type, create_date, page_index=1, page_size=20):
        """
        获取强平订单
        contract_code	true	string	品种代码		支持大小写，"BTC-USD"
        trade_type	true	int	交易类型		0:全部,5: 卖出强平,6: 买入强平
        create_date	true	int	日期		7，90（7天或者90天）
        page_index	false	int	页码,不填默认第1页	1
        page_size	false	int	不填默认20，不得多于50	20	[1-50]
        """
        params = package_data(remove_key(locals()))
        uri = self._create_coin_margined_api_uri("swap_liquidation_orders")
        return await self._request("get", uri, params=params)

    async def coin_margined_premium_index_kline(self, contract_code, period, size):
        """
        合约的溢价指数K线
        参数名称	是否必须	类型	描述	取值范围
        contract_code	true	string	合约代码	支持大小写，"BTC-USD" ...
        period	true	string	K线类型	1min, 5min, 15min, 30min, 60min,4hour,1day, 1week,1mon
        size	true	int	K线获取数量	[1,2000] （最多2000）
        """
        params = package_data(remove_key(locals()))
        uri = f"{self.API_FUTURE_URL}/index/market/history/swap_premium_index_kline"
        return await self._request("get", uri, params=params)

    async def coin_margined_estimated_rate_kline(self, contract_code, period, size):
        """
        实时预测资金费率的K线数据
        参数名称	是否必须	类型	描述	取值范围
        contract_code	true	string	合约代码	支持大小写，"BTC-USD" ...
        period	true	string	K线类型	1min, 5min, 15min, 30min, 60min,4hour,1day, 1week,1mon
        size	true	int	K线获取数量	[1,2000] （最多2000）
        """
        params = package_data(remove_key(locals()))
        uri = f"{self.API_FUTURE_URL}/index/market/history/swap_estimated_rate_kline"
        return await self._request("get", uri, params=params)

    async def coin_margined_basis_data(self, contract_code, period, basis_price_type=constants.BasisPriceType.OPEN, size=100):
        """
        获取基差数据
        contract_code	true	string	合约代码		如"BTC-USD"
        period	true	string	周期		仅支持小写，1min,5min, 15min, 30min, 60min,4hour,1day,1mon
        basis_price_type	false	string	基差价格类型，表示在周期内计算基差使用的价格类型	不填，默认使用开盘价	仅支持小写，开盘价：open，收盘价：close，最高价：high，最低价：low，平均价=（最高价+最低价）/2：average
        size	true	int	基差获取数量		[1,2000]
        """
        params = package_data(remove_key(locals()))
        uri = f"{self.API_FUTURE_URL}/index/market/history/swap_basis"
        return await self._request("get", uri, params=params)

    #### 币本位永续合约资产接口 #####
    async def coin_margined_balance_valuation(self, valuation_asset="BTC"):
        """
        获取账户总资产估值
        valuation_asset	false	string	资产估值币种，即按该币种为单位进行估值，不填默认"BTC"	"BTC","USD","CNY","EUR","GBP","VND","HKD","TWD","MYR","SGD","KRW","RUB","TRY"
        """
        data = {"valuation_asset": valuation_asset}
        return await self._request_coin_margined_private_api("post", "swap_balance_valuation", data=data)

    async def coin_margined_account_info(self, contract_code):
        """
        获取账户信息
        contract_code	false	string	支持大小写, "BTC-USD"... ,如果缺省，默认返回所有合约
        """
        data = package_data(remove_key(locals()))
        rst = await self._request_coin_margined_private_api('post', "swap_account_info", data=data)
        if rst.status == 'ok' and rst.data and contract_code:
            new_data = None
            for item in rst.data:
                if item['contract_code'].upper() == contract_code.upper():
                    new_data = item
                    break
            return Result(rst.code, rst.status, rst.msg, new_data)
        return rst

    async def coin_margined_position_info(self, contract_code=None):
        """
        获取用户持仓信息
        contract_code	false	string	支持大小写, "BTC-USD"... ,如果缺省，默认返回所有合约
        """
        data = package_data(remove_key(locals()))
        return await self._request_coin_margined_private_api("post", "swap_position_info", data=data)

    async def coin_margined_account_position_info(self, contract_code):
        """
        获取用户账户和持仓信息
        contract_code	false	string	支持大小写, "BTC-USD"...
        """
        data = package_data(remove_key(locals()))
        return await self._request_coin_margined_private_api("post", "swap_account_position_info", data=data)

    async def coin_margined_fee(self, contract_code=None):
        """
        查询用户当前的手续费费率
        contract_code	false	string	支持大小写, "BTC-USD"... ,如果缺省，默认返回所有合约
        """
        data = package_data(remove_key(locals()))
        return await self._request_coin_margined_private_api("post", "swap_fee", data=data)

    async def coin_margined_api_trading_status(self):
        """
        获取用户的API指标禁用信息
        """
        data = package_data(remove_key(locals()))
        return await self._request_coin_margined_private_api("get", "swap_api_trading_status", data=data)

    #### 币本位永续合约交易接口 #####
    async def coin_margined_create_order(
            self, direction, volume: int, offset, order_price_type, lever_rate=None, contract_code=None,
            client_order_id=None, price=None, tp_trigger_price=None, tp_order_price=None,
            tp_order_price_type=None, sl_trigger_price=None, sl_order_price=None, sl_order_price_type=None):
        """
        开单
        contract_code	string	true	合约代码,支持大小写,"BTC-USD"
        client_order_id	long	false	客户自己填写和维护，必须为数字, 请注意必须小于等于9223372036854775807
        price	decimal	false	价格
        volume	long	true	委托数量(张)
        direction	string	true	"buy":买 "sell":卖
        offset	string	true	"open":开 "close":平
        lever_rate	int	true	杠杆倍数[“开仓”若有10倍多单，就不能再下20倍多单;首次使用高倍杠杆(>20倍)，请使用主账号登录web端同意高倍杠杆协议后，才能使用接口下高倍杠杆(>20倍)]
        order_price_type	string	true	订单报价类型 "limit":限价，"opponent":对手价 ，"post_only":只做maker单,post only下单只受用户持仓数量限制,"optimal_5"：最优5档，"optimal_10：最优10档，"optimal_20"：最优20档，"fok":FOK订单，"ioc":IOC订单, opponent_ioc"： 对手价-IOC下单，"optimal_5_ioc"：最优5档-IOC下单，"optimal_10_ioc"：最优10档-IOC下单，"optimal_20_ioc"：最优20档-IOC下单,"opponent_fok"： 对手价-FOK下单，"optimal_5_fok"：最优5档-FOK下单，"optimal_10_fok"：最优10档-FOK下单，"optimal_20_fok"：最优20档-FOK下单
        tp_trigger_price	decimal	false	止盈触发价格
        tp_order_price	decimal	false	止盈委托价格（最优N档委托类型时无需填写价格）
        tp_order_price_type	string	false	止盈委托类型 不填默认为limit; 限价：limit ，最优5档：optimal_5，最优10档：optimal_10，最优20档：optimal_20
        sl_trigger_price	decimal	false	止损触发价格
        sl_order_price	decimal	false	止损委托价格（最优N档委托类型时无需填写价格）
        sl_order_price_type	string	false	止损委托类型 不填默认为limit; 限价：limit ，最优5档：optimal_5，最优10档：optimal_10，最优20档：optimal_20

        备注
        对手价下单price价格参数不用传，对手价下单价格是买一和卖一价,"optimal_5"：最优5档，"optimal_10：最优10档，"optimal_20"：最优20档下单price价格参数不用传，"limit":限价，"post_only":只做maker单 需要传价格。
        若存在持仓，那么下单时杠杆倍数必须与持仓杠杆相同，否则下单失败。若需使用新杠杆下单，则必须先使用切换杠杆接口将持仓杠杆切换成功后再下单。
        只有开仓订单才支持设置止盈止损。
        止盈触发价格为设置止盈单必填字段，止损触发价格为设置止损单必填字段；若缺省触发价格字段则不会设置对应的止盈单或止损单。

        开平方向
        开多：买入开多(direction用buy、offset用open)

        平多：卖出平多(direction用sell、offset用close)

        开空：卖出开空(direction用sell、offset用open)

        平空：买入平空(direction用buy、offset用close)
        """
        data = package_data(remove_key(locals()))
        return await self._request_coin_margined_private_api("post", "swap_order", data=data)

    async def coin_margined_batch_create_order(self, orders_data: list):
        """
        批量开单
        :param orders_data: 参数同开单接口
        :return:
        """
        data = package_data(remove_key(locals()))
        return await self._request_coin_margined_private_api("post", "swap_batchorder", data=data)

    async def coin_margined_cancel_order(self, contract_code, order_id=None, client_order_id=None):
        """
        撤销订单
        order_id	false (请看备注)	string	订单ID(多个订单ID中间以","分隔,一次最多允许撤消10个订单)
        client_order_id	false (请看备注)	string	客户订单ID(多个订单ID中间以","分隔,一次最多允许撤消10个订单)
        contract_code	true	string	合约代码,支持大小写,"BTC-USD"

        备注：
        order_id和client_order_id都可以用来撤单，至少要填写一个,同时只可以设置其中一种，如果设置了两种，默认以order_id来撤单。
        撤单接口返回结果只代表撤单命令发送成功，建议根据订单查询接口查询订单的状态来确定订单是否已真正撤销。
        client_order_id，8 小时有效，超过 8 小时的订单根据client_order_id将查询不到。
        """
        assert contract_code is not None
        data = package_data(remove_key(locals()))
        return await self._request_coin_margined_private_api("post", "swap_cancel", data=data)

    async def coin_margined_cancel_all(self, contract_code, direction=None, offset=None):
        """
        全部撤单
        contract_code	true	string	合约代码,支持大小写，"BTC-USD"
        direction	false	string	买卖方向（不填默认全部） "buy":买 "sell":卖
        offset	false	string	开平方向（不填默认全部） "open":开 "close":平

        备注
        direction与offset可只填其一，只填其一则按对应的条件去撤单。（如用户只传了direction=buy，则撤销所有买单，包括开仓和平仓）
        """
        data = package_data(remove_key(locals()))
        return await self._request_coin_margined_private_api("post", "swap_cancelall", data=data)

    async def coin_margined_switch_lever_rate(self, contract_code, lever_rate: int):
        """
        切换杠杆
        contract_code	true	string	合约代码	比如“BTC-USD

        备注
        只有在单个品种下只有持仓，且没有挂单的场景下，才可以切换该品种当前的倍数。
        接口限制请求次数为每3秒一次。
        """
        data = package_data(remove_key(locals()))
        return await self._request_coin_margined_private_api("post", "swap_switch_lever_rate", data=data)

    async def coin_margined_order_info(self, contract_code, order_id=None, client_order_id=None):
        """
        订单信息
        参数名称	是否必须	类型	描述
        order_id	false（请看备注）	string	订单ID(多个订单ID中间以","分隔,一次最多允许查询50个订单)
        client_order_id	false（请看备注）	string	客户订单ID(多个订单ID中间以","分隔,一次最多允许查询50个订单)
        contract_code	true	string	合约代码,支持大小写,"BTC-USD"

        最多只能查询 2 小时内的撤单信息。
        order_id和client_order_id至少要填写一个。
        order_id和client_order_id都可以用来查询，同时只可以设置其中一种，如果设置了两种，默认以order_id来查询。结算后，会把结束状态的订单（5部分成交已撤单 6全部成交 7已撤单）删除掉。
        client_order_id，8 小时有效，超过 8 小时的订单根据client_order_id将查询不到。
        """
        data = package_data(remove_key(locals()))
        return await self._request_coin_margined_private_api("post", "swap_order_info", data=data)

    async def coin_margined_order_detail(self, contract_code, order_id, create_at: int = None, order_type=None,
                                         page_index=1, page_size=20):
        """
        订单明细信息
        参数名称	是否必须	类型	描述
        contract_code	true	string	合约代码,支持大小写,"BTC-USD"
        order_id	true	long	订单id
        created_at	false	long	下单时间戳
        order_type	false	int	订单类型，1:报单 、 2:撤单 、 3:强平、4:交割
        page_index	false	int	第几页,不填第一页
        page_size	false	int	不填默认20，不得多于50

        获取订单明细接口查询撤单数据时，如果传“created_at”和“order_type”参数则能查询最近6小时数据，如果不传“created_at”和“order_type”参数只能查询到最近2小时数据。
        order_id返回是18位，nodejs和javascript默认解析18有问题，nodejs和javascript里面JSON.parse默认是int，超过18位的数字用json-bigint的包解析。
        created_at使用13位long类型时间戳（包含毫秒时间），如果输入准确的时间戳，查询性能将会提升。例如:"2019/10/18 10:26:22"转换为时间戳为：1571365582123。也可以直接从contract_order下单接口返回的ts中获取时间戳查询对应的订单。
        created_at禁止传0。
        """
        data = package_data(remove_key(locals()))
        return await self._request_coin_margined_private_api("post", "swap_order_detail", data=data)

    async def coin_margined_open_orders(self, contract_code, page_index=1, page_size=20, sort_by="created_at", trade_type=0):
        """
        获取合约当前未成交委托

        参数名称	是否必须	类型	描述	默认值	取值范围
        contract_code	true	string	合约代码	支持大小写,"BTC-USD" ...
        page_index	false	int	页码，不填默认第1页	1
        page_size	false	int			不填默认20，不得多于50
        sort_by	false	string	排序字段，不填默认按创建时间倒序	created_at	“created_at”(按照创建时间倒序)，“update_time”(按照更新时间倒序)
        trade_type	false	int	交易类型，不填默认查询全部	0	0:全部,1:买入 开多,2: 卖出开空,3: 买入平空,4: 卖出平多。
        """
        data = package_data(remove_key(locals()))
        return await self._request_coin_margined_private_api("post", "swap_openorders", data=data)

    async def coin_margined_match_result(self, contract_code, trade_type, create_date, page_index=1, page_size=20):
        """
        获取历史成交记录

        参数名称	是否必须	类型	描述	默认值	取值范围
        contract_code	true	string	合约代码	支持大小写，"BTC-USD" ...
        trade_type	true	int	交易类型		0:全部,1:买入开多,2: 卖出开空,3: 买入平空,4: 卖出平多,5: 卖出强平,6: 买入强平
        create_date	true	int	日期		可随意输入正整数，如果参数超过90则默认查询90天的数据
        page_index	false	int	页码，不填默认第1页	1
        page_size	false	int	不填默认20，不得多于50	20
        """
        data = package_data(remove_key(locals()))
        return await self._request_coin_margined_private_api("post", "swap_matchresults", data=data)

    async def coin_margined_lightning_close_position(self, contract_code, volume, direction, client_order_id=None,
                                                     order_price_type=constants.OrderPriceType.LIGHTNING):
        """
        闪电平仓
        """
        data = package_data(remove_key(locals()))
        return await self._request_coin_margined_private_api("post", "swap_lightning_close_position", data=data)

    #### U本位永续合约交易接口 #####
    async def usd_margined_cancel_order(self, contract_code, order_id=None, client_order_id=None, cross=True):
        """
        【全仓】撤销订单
        """
        assert contract_code is not None
        data = package_data(remove_key(locals(), "self,cross"))
        return await self._request_usd_margined_private_api(
            "post", "swap_cross_cancel" if cross else "swap_cancel", data=data
        )





























import timeit
import os
import asyncio

import aiohttp.helpers

from binance.client import Client, AsyncClient

loop = 3

API_KEY = os.environ.get("binance_quant_api_key")
API_SECRET = os.environ.get("binance_quant_api_secret")

symbol = "BUSDUSDT"


async def async_create_order():
    client = await AsyncClient.create(API_KEY, API_SECRET)

    async def task():
        resp = await client.order_limit_buy(symbol=symbol, price="0.95", quantity=11)
        await client.cancel_order(symbol=symbol, orderId=resp['orderId'])

    start = timeit.default_timer()
    await asyncio.gather(*[task() for _ in range(loop)])
    end = timeit.default_timer()
    await client.close_connection()
    print(f"异步版本: {end - start}")


def create_order():
    client = Client(API_KEY, API_SECRET)
    start = timeit.default_timer()

    for _ in range(loop):
        resp = client.order_limit_buy(symbol=symbol, price="0.95", quantity=11)
        client.cancel_order(symbol=symbol, orderId=resp['orderId'])
    end = timeit.default_timer()
    print(f"同步版本: {end-start}")


if __name__ == '__main__':
    # create_order()
    asyncio.run(async_create_order())



import aiohttp


class HuobiAPIException(Exception):
    def __init__(self, response: aiohttp.ClientResponse, error_msg: str, error_code=None):
        self.message = error_msg
        self.status_code = response.status
        self.response = response
        self.request = getattr(response, 'request', None)
        self.error_code = error_code

    def __str__(self):  # pragma: no cover
        return f'HuobiAPIError({self.error_code}): {self.message}'



import base64
import hmac
from hashlib import sha256
from urllib import parse


def generate_signature(api_secret: str, method: str, uri: str, params: dict) -> str:
    url_path_result = parse.urlparse(uri)
    sorted_params = sorted(params.items(), key=lambda d: d[0], reverse=False)
    encode_params = parse.urlencode(sorted_params)
    payload = [method.upper(), url_path_result.netloc, url_path_result.path, encode_params]
    payload = "\n".join(payload)
    payload = payload.encode(encoding="UTF8")
    secret_key = api_secret.encode(encoding="utf8")
    digest = hmac.new(secret_key, payload, digestmod=sha256).digest()
    signature = base64.b64encode(digest)
    signature = signature.decode()
    return signature



import base64
import hmac
import json
from hashlib import sha256


def generate_signature(method: str, request_path: str, timestamp: str, params, body, secret_key) -> str:
    if body is None or body == {}:
        body = ''
    else:
        body = json.dumps(body)

    if method.upper() == "GET":
        if params:
            request_path = request_path + "?" + "&".join([f"{k}={v}" for k, v in params.items()])

    msg = timestamp + method.upper() + request_path + body
    digest = hmac.new(bytes(secret_key, encoding='utf8'), bytes(msg, encoding='utf8'), digestmod=sha256).digest()
    return base64.b64encode(digest).decode('utf-8')


def create_ws_payload(channel: str, payload: dict):
    payload['channel'] = channel
    return payload



class TradeType:
    SPOT = "spot"
    COIN_M = "coin_m"
    USD_M = "usd_m"


class OrderUpdateEvent:
    NEW = "NEW"
    CANCELED = "CANCELED"
    CALCULATED = "CALCULATED"
    EXPIRED = "EXPIRED"
    TRADE = "TRADE"



"""
This module contains the set of AssetMove' exceptions.
"""


class BasicException(Exception):
    def __init__(self, *args, **kwargs):
        super(BasicException, self).__init__(*args, **kwargs)


class SystemUnNormal(BasicException):
    """系统不可用,维护中"""


class InsufficientBalance(BasicException):
    """账户余额不足"""


class TradePairUnavailable(BasicException):
    """交易对维护"""


class UnknownError(BasicException):
    """未知错误"""


class HuoBiWebSocketError(BasicException):
    """WS错误"""


class OkexWebSocketError(BasicException):
    """WS错误"""



import os


class Huobi:
    API_KEY = os.environ.get("huobi_main_api_key")
    API_SECRET = os.environ.get("huobi_main_api_secret")


class Binance:
    API_KEY = os.environ.get("binance_quant_api_key")
    API_SECRET = os.environ.get("binance_quant_api_secret")


class Okex:
    API_KEY = os.environ.get("okex_simulated_api_key")
    API_SECRET = os.environ.get("okex_simulated_api_secret")
    API_PASSPHRASE = os.environ.get("okex_simulated_api_passphrase")



from binance.client import Client, AsyncClient


class Wallet:
    def __init__(self, client: Client = None, async_client: AsyncClient = None):
        self.client = client
        self.async_client = async_client

    def get_spot_balance(self, asset: str = None) -> dict:
        """
        现货账户余额
        :param asset:
        :return:
        {
            "asset": "KMD",
            "free": "0.00000000",
            "locked": "0.00000000"
        }
        """
        rst = self.client.get_account()
        # list -> dict
        balances = {item['asset']: item for item in rst['balances']}
        if asset:
            return balances[asset.upper()]
        return balances

    async def async_get_spot_balance(self, asset: str = None) -> dict:
        rst = await self.async_client.get_account()
        # list -> dict
        balances = {item['asset']: item for item in rst['balances']}
        if asset:
            return balances[asset.upper()]
        return balances

    def get_coin_future_balance(self, asset: str = None) -> dict:
        """
        币本位账户余额
        {
            "accountAlias": "SgsR",    // 账户唯一识别码
            "asset": "BTC",     // 资产
            "balance": "0.00250000",    // 账户余额
            "withdrawAvailable": "0.00250000", // 最大可提款金额,同`GET /dapi/account`中"maxWithdrawAmount"
            "crossWalletBalance": "0.00241969", // 全仓账户余额
            "crossUnPnl": "0.00000000", // 全仓持仓未实现盈亏
            "availableBalance": "0.00241969",    // 可用下单余额
            "updateTime": 1592468353979
        }
        """
        rst = self.client.futures_coin_account_balance()
        # list -> dict
        balances = {item['asset']: item for item in rst}

        if asset:
            return balances[asset]
        return balances

    async def async_get_coin_future_balance(self, asset: str = None) -> dict:
        rst = await self.async_client.futures_coin_account_balance()
        # list -> dict
        balances = {item['asset']: item for item in rst}

        if asset:
            return balances[asset]
        return balances




import asyncio
import json
import gzip
import logging
import traceback
import uuid
from datetime import datetime
from typing import Optional
from functools import partial, wraps
from urllib.parse import urlparse

import aiohttp
from aiohttp.web_ws import WSMessage

from common.exceptions import HuoBiWebSocketError
from common.huobi import constants
from common.huobi.constants import WebSocketOP
from common.huobi.tools import generate_signature


def is_coin_margined_market(f):
    @wraps(f)
    async def wrapper(self, *args, **kwargs):
        assert self.ws.url_path == HuoBiWebSocketManager.COIN_MARGINED_MARKET
        return await f(self, *args, **kwargs)
    return wrapper


def is_coin_margined_order(f):
    @wraps(f)
    async def wrapper(self, *args, **kwargs):
        assert self.ws.url_path == HuoBiWebSocketManager.COIN_MARGINED_ORDER
        return await f(self, *args, **kwargs)
    return wrapper


def is_future_market(f):
    @wraps(f)
    async def wrapper(self, *args, **kwargs):
        assert self.ws.url_path == HuoBiWebSocketManager.FUTURE_MARKET
        return await f(self, *args, **kwargs)
    return wrapper


def is_future_order(f):
    @wraps(f)
    async def wrapper(self, *args, **kwargs):
        assert self.ws.url_path == HuoBiWebSocketManager.FUTURE_ORDER
        return await f(self, *args, **kwargs)
    return wrapper


def is_index(f):
    @wraps(f)
    async def wrapper(self, *args, **kwargs):
        assert self.ws.url_path == HuoBiWebSocketManager.INDEX_DATA
        return await f(self, *args, **kwargs)
    return wrapper


def is_system_status(f):
    @wraps(f)
    async def wrapper(self, *args, **kwargs):
        assert self.ws.url_path == HuoBiWebSocketManager.SYSTEM_STATUS
        return await f(self, *args, **kwargs)
    return wrapper


class ReconnectingWebSocket:
    TIMEOUT = 20

    def __init__(self, url: str, auto_ping=True, proxy=None,
                 api_key: Optional[str] = None, api_secret: Optional[str] = None):
        """
        @param url 建立websocket的地址
        """
        self._url = url
        self.url_path = urlparse(url).path[1:]
        self.ws: Optional[aiohttp.ClientWebSocketResponse] = None
        self._session: Optional[aiohttp.ClientSession] = None
        self._conn = None
        self._proxy = proxy
        self._auto_ping = auto_ping
        self._api_key = api_key
        self._api_secret = api_secret
        self._connected = False
        self._already_send_auth = False
        self._complete_auth = False
        self._sub_channels = []
        self.auth_complete_event = asyncio.Event()

    @property
    def api_key(self):
        return self._api_key

    @property
    def need_send_auth(self) -> bool:
        return self._api_secret and self._api_key and not self._already_send_auth

    @property
    def need_reconnected(self) -> bool:
        return self.ws is None or self.ws.closed or self._connected is False

    def _reset_flag(self):
        """
        连接断开后重置标志变量
        """
        # 断开后conn已经失效，必须重连
        self._conn = None
        self._connected = self._already_send_auth = self._complete_auth = False

    async def send_auth(self):
        if not self.ws:
            raise HuoBiWebSocketError("no binding web socket")
        if self.ws.closed:
            raise HuoBiWebSocketError("websocket has closed")

        timestamp = datetime.utcnow().strftime("%Y-%m-%dT%H:%M:%S")
        data = {
            "AccessKeyId": self._api_key,
            "SignatureMethod": "HmacSHA256",
            "SignatureVersion": "2",
            "Timestamp": timestamp
        }
        sign = generate_signature(self._api_secret, "GET", self._url, data)
        data['op'] = WebSocketOP.AUTH
        data['type'] = 'api'
        data['Signature'] = sign
        logging.debug(f"send auth msg: {data}")
        await self.ws.send_json(data)
        self._already_send_auth = True

    async def __aenter__(self):
        if self.need_reconnected:
            self._reset_flag()
            await self._reconnect()
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        if self.ws:
            await self.ws.close()
        if self._conn:
            await self._conn.__aexit__(exc_type, exc_val, exc_tb)
        self.ws = None
        self._reset_flag()
        if self._session:
            await self._session.close()
            self._session = None

    async def _reconnect(self):
        logging.warning("reconnecting web socket right now!")
        await self._connect()
        await self._connected_callback()

    async def _connect(self):
        logging.info(f"ws connect {self._url}")
        if not self._session or self._session.closed:
            self._session = aiohttp.ClientSession(trust_env=True)
            self._conn = None
            self.ws = None
        if self._conn is None:
            self._conn = self._session.ws_connect(self._url, proxy=self._proxy)
            self.ws = None
        try:
            if self.ws is None:
                self.ws = await self._conn.__aenter__()
            self._connected = True
            logging.debug("connect successfully!")
        except aiohttp.client.ClientConnectorError:
            logging.error(f"connect to ws server error! url: {self._url}")

    async def _connected_callback(self):
        """
        After connect to WebSocket server successfully, send a auth message to server.
        """
        if self.need_send_auth:
            await self.send_auth()
            logging.debug("wait for auth response")
            while self._complete_auth is False:
                await self.recv()
            self.auth_complete_event.set()
            logging.debug("auth complete!")
        for payload in self._sub_channels:
            await self.ws.send_json(payload)

    async def check_connection(self, *args, **kwargs):
        if not self.ws:
            logging.warning("web socket connection not connected yet!")
            return
        if self.ws.closed:
            self._reset_flag()
            await self._reconnect()

    async def recv(self):
        res = None
        if self.ws is None:
            raise HuoBiWebSocketError("web socket connection not connected yet")

        while not res:
            if self.ws.closed:
                self._reset_flag()
                await self._reconnect()
                continue
            try:
                res = await asyncio.wait_for(self.ws.receive(), timeout=self.TIMEOUT)
            except asyncio.TimeoutError:
                logging.error(f"no message in {self.TIMEOUT} seconds")
            except asyncio.CancelledError as e:
                logging.error(f"cancelled error {e}")
            except asyncio.IncompleteReadError as e:
                logging.error(f"incomplete read error {e}")
            except Exception as e:
                logging.error(f"exception {e}")
            else:
                res = self._handle_message(res)

            if isinstance(res, dict):
                if ("ping" in res or res.get("op") == WebSocketOP.PING) and self._auto_ping:
                    if self.url_path in (HuoBiWebSocketManager.FUTURE_ORDER, HuoBiWebSocketManager.COIN_MARGINED_ORDER,
                                         HuoBiWebSocketManager.USDT_MARGINED_ORDER):
                        pong_msg = {"op": WebSocketOP.PONG, "ts": res['ts']}
                    else:
                        pong_msg = {"pong": res['ping']}
                    await self.ws.send_json(pong_msg)
                    logging.debug(f"send {pong_msg}")
                    res = None
                elif res.get("op") == WebSocketOP.CLOSE:
                    self._reset_flag()
                    await self._reconnect()
                    res = None
                elif res.get("op") == WebSocketOP.AUTH and res.get('err-code') == 0:
                    self._complete_auth = True

                if res and res.get('err-code'):
                    logging.error(res)

        return res

    def _handle_message(self, msg: WSMessage):
        if msg.type == aiohttp.WSMsgType.TEXT:
            try:
                data = json.loads(msg.data)
            except:
                data = msg.data
        elif msg.type == aiohttp.WSMsgType.BINARY:
            uncompress_data = gzip.decompress(msg.data).decode()
            try:
                data = json.loads(uncompress_data)
            except:
                data = uncompress_data
        elif msg.type in (aiohttp.WSMsgType.CLOSE, aiohttp.WSMsgType.CLOSED, aiohttp.WSMsgType.ERROR):
            logging.error(f"receive unexpected msg: {msg}")
            return None
        else:
            raise HuoBiWebSocketError(f"invalid msg type: {msg.type}")

        logging.debug(f"receive msg: {data}")
        return data

    async def send_heartbeat_msg(self, msg, *args, **kwargs):
        """
        发送心跳给server
        """
        if not self.ws:
            logging.warning("web socket connection not connected yet!")
        try:
            if isinstance(msg, dict):
                await self.ws.send_json(msg)
            elif isinstance(msg, str):
                await self.ws.send_str(msg)
            else:
                logging.error(f"invalid heartbeat msg type: {msg}")
            logging.debug(f"send ping message: {msg}")
        except ConnectionResetError:
            traceback.print_exc()
            await asyncio.get_running_loop().create_task(self._reconnect())

    async def send_request(self, payload):
        if self._connected is False or self.ws is None or self.ws.closed:
            await self._connect()
            await self._connected_callback()
        # payload['id'] = str(uuid.uuid1())
        logging.debug(f"send msg: {payload}")
        await self.ws.send_json(payload)

    async def sub(self, payload):
        if payload not in self._sub_channels:
            self._sub_channels.append(payload)
        await self.send_request(payload)

    async def unsub(self, payload):
        if payload in self._sub_channels:
            self._sub_channels.remove(payload)
        await self.send_request(payload)


class HuoBiWebSocketManager:
    BASE_URL = "wss://api.hbdm.com/"
    BASE_AWS_URL = "wss://api.hbdm.vn/"

    FUTURE_MARKET = "ws"
    FUTURE_ORDER = "notification"
    COIN_MARGINED_MARKET = "swap-ws"
    COIN_MARGINED_ORDER = "swap-notification"
    USDT_MARGINED_MARKET = "linear-swap-ws"
    USDT_MARGINED_ORDER = "linear-swap-notification"

    INDEX_DATA = "ws_index"     # 指数/基差
    SYSTEM_STATUS = "center-notification"

    def __init__(self, api_key: Optional[str] = None, api_secret: Optional[str] = None, proxy=None):
        self.proxy = proxy
        self.ws: Optional[ReconnectingWebSocket] = None
        self._api_key = api_key
        self._api_secret = api_secret
        self._create_ws = partial(ReconnectingWebSocket, proxy=self.proxy, api_key=api_key, api_secret=api_secret)

    def set_coin_margined_market_socket(self) -> "HuoBiWebSocketManager":
        self.ws = self._create_ws(url=self.BASE_URL + self.COIN_MARGINED_MARKET)
        return self

    def set_coin_margined_order_socket(self) -> "HuoBiWebSocketManager":
        self.ws = self._create_ws(url=self.BASE_URL + self.COIN_MARGINED_ORDER)
        return self

    def set_future_market_socket(self) -> "HuoBiWebSocketManager":
        self.ws = self._create_ws(url=self.BASE_URL + self.FUTURE_MARKET)
        return self

    def set_future_order_socket(self) -> "HuoBiWebSocketManager":
        self.ws = self._create_ws(url=self.BASE_URL + self.FUTURE_ORDER)
        return self

    def set_index_socket(self) -> "HuoBiWebSocketManager":
        self.ws = self._create_ws(url=self.BASE_URL + self.INDEX_DATA)
        return self

    def set_system_status_socket(self) -> "HuoBiWebSocketManager":
        self.ws = self._create_ws(url=self.BASE_URL + self.SYSTEM_STATUS)
        return self

    ######################################################
    # 币本位永续合约接口                                    #
    ######################################################

    ##########
    # 行情
    ##########

    @is_coin_margined_market
    async def sub_coin_margined_kline(self, symbol: str, period=constants.KLINE_INTERVAL_1MINUTE):
        await self.ws.send_request({WebSocketOP.SUB: f"market.{symbol}.kline.{period}"})

    @is_coin_margined_market
    async def req_coin_margined_kline(self, symbol: str, start: int, end: int, period=constants.KLINE_INTERVAL_1MINUTE):
        """
        :param symbol:
        :param start:
        :param end: 开始时间/秒
        :param period: 结束时间/秒
        :return:
        """
        await self.ws.send_request({
            WebSocketOP.REQ: f"market.{symbol.upper()}.kline.{period}",
            "from": start,
            "to": end
        })

    @is_coin_margined_market
    async def sub_coin_margined_market_depth(self, symbol: str, depth_type=constants.WS_DEPTH_5):
        await self.ws.send_request({WebSocketOP.SUB: f"market.{symbol}.depth.{depth_type}"})

    @is_coin_margined_market
    async def sub_coin_margined_incremental_market_depth(self, symbol: str, size=20, data_type=constants.DataType.INCREMENTAL):
        await self.ws.send_request({
            WebSocketOP.SUB: f"market.{symbol}.depth.size_{size}.high_freq",
            "data_type": data_type
        })

    @is_coin_margined_market
    async def sub_coin_margined_market_detail(self, symbol: str):
        await self.ws.send_request({
            WebSocketOP.SUB: f"market.{symbol}.detail"
        })

    @is_coin_margined_market
    async def sub_coin_margined_bbo(self, symbol: str):
        await self.ws.send_request({
            WebSocketOP.SUB: f"market.{symbol}.bbo"
        })

    @is_coin_margined_market
    async def sub_coin_margined_trade_detail(self, symbol: str):
        await self.ws.send_request({WebSocketOP.SUB: f"market.{symbol}.trade.detail"})

    @is_coin_margined_market
    async def req_coin_margined_trade_detail(self, symbol: str):
        await self.ws.send_request({WebSocketOP.REQ: f"market.{symbol}.trade.detail"})

    #############
    # 订单/用户数据
    #############

    @is_coin_margined_order
    async def sub_coin_margined_order(self, symbol):
        await self.ws.send_request({
            "op": WebSocketOP.SUB,
            "topic": f"orders.{symbol}"
        })

    @is_coin_margined_order
    async def unsub_coin_margined_order(self, symbol):
        await self.ws.send_request({
            "op": WebSocketOP.UNSUB,
            "topic": f"orders.{symbol}"
        })

    @is_coin_margined_order
    async def sub_coin_margined_account(self, symbol):
        await self.ws.send_request({
            "op": WebSocketOP.SUB,
            "topic": f"accounts.{symbol}"
        })

    @is_coin_margined_order
    async def unsub_coin_margined_account(self, symbol):
        await self.ws.send_request({
            "op": WebSocketOP.UNSUB,
            "topic": f"accounts.{symbol}"
        })

    @is_coin_margined_order
    async def sub_coin_margined_position(self, symbol):
        await self.ws.send_request({
            "op": WebSocketOP.SUB,
            "topic": f"positions.{symbol}"
        })

    @is_coin_margined_order
    async def unsub_coin_margined_position(self, symbol):
        await self.ws.send_request({
            "op": WebSocketOP.UNSUB,
            "topic": f"positions.{symbol}"
        })

    @is_coin_margined_order
    async def sub_coin_margined_match_orders(self, symbol):
        """订单撮合数据"""
        await self.ws.send_request({
            "op": WebSocketOP.SUB,
            "topic": f"matchOrders.{symbol}"
        })

    @is_coin_margined_order
    async def unsub_coin_margined_match_orders(self, symbol):
        await self.ws.send_request({
            "op": WebSocketOP.UNSUB,
            "topic": f"matchOrders.{symbol}"
        })

    @is_coin_margined_order
    async def sub_coin_margined_liquidation_orders(self, symbol):
        """强平订单数据"""
        await self.ws.send_request({
            "op": WebSocketOP.SUB,
            "topic": f"public.{symbol}.liquidation_orders"
        })

    @is_coin_margined_order
    async def unsub_coin_margined_liquidation_orders(self, symbol):
        await self.ws.send_request({
            "op": WebSocketOP.UNSUB,
            "topic": f"public.{symbol}.liquidation_orders"
        })

    @is_coin_margined_order
    async def sub_coin_margined_funding_rate(self, symbol):
        """资金费率推送"""
        await self.ws.send_request({
            "op": WebSocketOP.SUB,
            "topic": f"public.{symbol}.funding_rate"
        })

    @is_coin_margined_order
    async def unsub_coin_margined_funding_rate(self, symbol):
        await self.ws.send_request({
            "op": WebSocketOP.UNSUB,
            "topic": f"public.{symbol}.funding_rate"
        })

    @is_coin_margined_order
    async def sub_coin_margined_contract_info(self, symbol):
        """合约信息变动"""
        await self.ws.send_request({
            "op": WebSocketOP.SUB,
            "topic": f"public.{symbol}.contract_info"
        })

    @is_coin_margined_order
    async def unsub_coin_margined_contract_info(self, symbol):
        await self.ws.send_request({
            "op": WebSocketOP.UNSUB,
            "topic": f"public.{symbol}.contract_info"
        })

    @is_coin_margined_order
    async def sub_coin_margined_trigger_order(self, symbol):
        """计划委托订单更新"""
        await self.ws.send_request({
            "op": WebSocketOP.SUB,
            "topic": f"trigger_order.{symbol}"
        })

    @is_coin_margined_order
    async def unsub_coin_margined_trigger_order(self, symbol):
        await self.ws.send_request({
            "op": WebSocketOP.UNSUB,
            "topic": f"trigger_order.{symbol}"
        })

    ########################
    ### 交割合约接口
    ########################

    #### 行情 #####
    @is_future_market
    async def sub_future_kline(self, symbol: str, period=constants.KLINE_INTERVAL_1MINUTE):
        """
        KLine数据
        symbol	true	string	交易对		支持大小写，如"BTC_CW"表示BTC当周合约，"BTC_NW"表示BTC次周合约，"BTC_CQ"表示BTC当季合约 , "BTC_NQ"表示次季度合约 。支持使用合约code来订阅 例如："BTC200918"(当周)，"BTC200925"(次周)，"BTC201225"(季度)，"BTC210326"(次季度)。
        period	true	string	K线周期    仅支持小写，1min, 5min, 15min, 30min, 60min,4hour,1day,1week, 1mon
        """
        await self.ws.send_request({WebSocketOP.SUB: f"market.{symbol}.kline.{period}"})

    @is_future_market
    async def req_future_kline(self, symbol: str, start: int, end: int, period=constants.KLINE_INTERVAL_1MINUTE):
        await self.ws.send_request({
            WebSocketOP.REQ: f"market.{symbol.upper()}.kline.{period}",
            "from": start,
            "to": end
        })

    @is_future_market
    async def sub_future_market_depth(self, symbol: str, depth_type=constants.WS_DEPTH_5):
        """
        深度
        symbol	true	string	交易对		支持大小写，如"BTC_CW"表示BTC当周合约，"BTC_NW"表示BTC次周合约，"BTC_CQ"表示BTC当季合约, "BTC_NQ"表示次季度合约". 支持使用合约code来订阅 例如："BTC200918"(当周)，"BTC200925"(次周)，"BTC201225"(季度)，"BTC210326"(次季度)。
        type	true	string	Depth 类型		获得150档深度数据，使用step0, step1, step2, step3, step4, step5, step14, step15 （step1至step15是进行了深度合并后的深度），使用step0时，不合并深度获取150档数据;获得20档深度数据，使用 step6, step7, step8, step9, step10, step11, step12, step13（step7至step13是进行了深度合并后的深度），使用step6时，不合并深度获取20档数据
        """
        await self.ws.send_request({WebSocketOP.SUB: f"market.{symbol}.depth.{depth_type}"})

    @is_future_market
    async def sub_future_incremental_market_depth(self, symbol: str, size=20, data_type=constants.DataType.INCREMENTAL):
        """
        深度增量数据
        :param symbol:
        :param size:
        :param data_type:
        :return:
        """
        await self.ws.send_request({
            WebSocketOP.SUB: f"market.{symbol}.depth.size_{size}.high_freq",
            "data_type": data_type
        })

    @is_future_market
    async def sub_future_market_detail(self, symbol: str):
        await self.ws.send_request({
            WebSocketOP.SUB: f"market.{symbol}.detail"
        })

    @is_future_market
    async def sub_future_bbo(self, symbol: str):
        """
        买一卖一逐笔行情数据
        :param symbol:
        :return:
        """
        await self.ws.send_request({
            WebSocketOP.SUB: f"market.{symbol}.bbo"
        })

    @is_future_market
    async def sub_future_trade_detail(self, symbol: str):
        await self.ws.send_request({WebSocketOP.SUB: f"market.{symbol}.trade.detail"})

    @is_future_market
    async def req_future_trade_detail(self, symbol: str, size=50):
        await self.ws.send_request({
            WebSocketOP.REQ: f"market.{symbol}.trade.detail",
            "size": size
        })

    # 订单/资产变动
    @is_future_order
    async def sub_future_order(self, symbol):
        """
        订单成交数据
        :param symbol:
        :return:
        """
        await self.ws.send_request({
            "op": WebSocketOP.SUB,
            "topic": f"orders.{symbol}"
        })

    @is_future_order
    async def unsub_future_order(self, symbol):
        await self.ws.send_request({
            "op": WebSocketOP.UNSUB,
            "topic": f"orders.{symbol}"
        })

    @is_future_order
    async def sub_future_match_orders(self, symbol):
        """订单撮合数据"""
        await self.ws.send_request({
            "op": WebSocketOP.SUB,
            "topic": f"matchOrders.{symbol}"
        })

    @is_future_order
    async def unsub_future_match_orders(self, symbol):
        await self.ws.send_request({
            "op": WebSocketOP.UNSUB,
            "topic": f"matchOrders.{symbol}"
        })

    @is_future_order
    async def sub_future_account(self, symbol):
        """
        资产变动
        :param symbol:
        :return:
        """
        await self.ws.send_request({
            "op": WebSocketOP.SUB,
            "topic": f"accounts.{symbol}"
        })

    @is_future_order
    async def unsub_future_account(self, symbol):
        await self.ws.send_request({
            "op": WebSocketOP.UNSUB,
            "topic": f"accounts.{symbol}"
        })

    @is_future_order
    async def sub_future_position(self, symbol):
        """
        持仓变动
        :param symbol:
        :return:
        """
        await self.ws.send_request({
            "op": WebSocketOP.SUB,
            "topic": f"positions.{symbol}"
        })

    @is_future_order
    async def unsub_future_position(self, symbol):
        await self.ws.send_request({
            "op": WebSocketOP.UNSUB,
            "topic": f"positions.{symbol}"
        })

    @is_future_order
    async def sub_future_liquidation_orders(self, symbol):
        """强平订单数据"""
        await self.ws.send_request({
            "op": WebSocketOP.SUB,
            "topic": f"public.{symbol}.liquidation_orders"
        })

    @is_future_order
    async def unsub_future_liquidation_orders(self, symbol):
        await self.ws.send_request({
            "op": WebSocketOP.UNSUB,
            "topic": f"public.{symbol}.liquidation_orders"
        })

    @is_future_order
    async def sub_future_contract_info(self, symbol):
        """合约信息变动"""
        await self.ws.send_request({
            "op": WebSocketOP.SUB,
            "topic": f"public.{symbol}.contract_info"
        })

    @is_future_order
    async def unsub_future_contract_info(self, symbol):
        await self.ws.send_request({
            "op": WebSocketOP.UNSUB,
            "topic": f"public.{symbol}.contract_info"
        })

    @is_future_order
    async def sub_future_trigger_order(self, symbol):
        """计划委托订单更新"""
        await self.ws.send_request({
            "op": WebSocketOP.SUB,
            "topic": f"trigger_order.{symbol}"
        })

    @is_future_order
    async def unsub_future_trigger_order(self, symbol):
        await self.ws.send_request({
            "op": WebSocketOP.UNSUB,
            "topic": f"trigger_order.{symbol}"
        })


    ########################
    ### 指数/基差数据
    ########################
    @is_index
    async def sub_index(self, symbol, period=constants.KLINE_INTERVAL_1HOUR):
        """
        K线数据
        :param symbol:  支持大小写，"BTC-USD","ETH-USD"...
        :param period: 仅支持小写,1min, 5min, 15min, 30min, 60min,4hour,1day, 1mon
        :return:
        """
        await self.ws.send_request({
            WebSocketOP.SUB: f"market.{symbol}.index.{period}"
        })

    @is_index
    async def req_index(self, symbol, start: int, end: int, period=constants.KLINE_INTERVAL_1HOUR):
        """
        请求K线数据
        :param symbol:
        :param start:
        :param end:
        :param period:
        :return:
        """
        await self.ws.send_request({
            WebSocketOP.REQ: f"market.{symbol}.index.{period}",
            "from": start,
            "to": end
        })

    @is_index
    async def sub_premium_index(self, symbol, period=constants.KLINE_INTERVAL_1DAY):
        """
        溢价K线数据
        :param symbol:
        :param period:
        :return:
        """
        await self.ws.send_request({
            WebSocketOP.SUB: f"market.{symbol}.premium_index.{period}"
        })

    @is_index
    async def req_premium_index(self, symbol, start: int, end: int, period=constants.KLINE_INTERVAL_1DAY):
        """
        请求溢价K线数据
        :param symbol:
        :param start:
        :param end:
        :param period:
        :return:
        """
        await self.ws.send_request({
            WebSocketOP.REQ: f"market.{symbol}.premium_index.{period}",
            "from": start,
            "to": end
        })

    @is_index
    async def sub_estimated_funding_rate(self, symbol, period=constants.KLINE_INTERVAL_60MINUTE):
        """
        预测基金费率K线
        """
        await self.ws.send_request({
            WebSocketOP.SUB: f"market.{symbol}.estimated_rate.{period}"
        })

    @is_index
    async def req_estimated_funding_rate(self, symbol, start: int, end: int, period=constants.KLINE_INTERVAL_60MINUTE):
        await self.ws.send_request({
            WebSocketOP.REQ: f"market.{symbol}.estimated_rate.{period}",
            "from": start,
            "to": end
        })

    @is_index
    async def sub_basis_data(self, symbol, period=constants.KLINE_INTERVAL_1DAY,
                                         price_type=constants.BasisPriceType.OPEN):
        """
        基差数据
        """
        await self.ws.send_request({
            WebSocketOP.SUB: f"market.{symbol}.basis.{period}.{price_type}"
        })

    @is_index
    async def req_basis_data(self, symbol, start: int, end: int, period=constants.KLINE_INTERVAL_1DAY,
                                         price_type=constants.BasisPriceType.OPEN):
        await self.ws.send_request({
            WebSocketOP.REQ: f"market.{symbol}.basis.{period}.{price_type}",
            "from": start,
            "to": end
        })

    @is_index
    async def sub_mark_price(self, symbol, period=constants.KLINE_INTERVAL_1DAY):
        """
        标记价格K线
        """
        await self.ws.send_request({
            WebSocketOP.SUB: f"market.{symbol}.mark_price.{period}"
        })

    @is_index
    async def req_mark_price(self, symbol, start: int, end: int, period=constants.KLINE_INTERVAL_1DAY):
        await self.ws.send_request({
            WebSocketOP.REQ: f"market.{symbol}.mark_price.{period}",
            "from": start,
            "to": end
        })

    ########################
    ### 系统状态
    ########################
    @is_system_status
    async def sub_system_status(self, service: str = "swap"):
        """
        :param service:
        swap - 币本位永续
        linear-swap - U本位永续
        futures - 交割
        :return:
        """
        await self.ws.send_request({
            "op": WebSocketOP.SUB,
            "topic": f"public.{service}.heartbeat"
        })

















from enum import Enum


class TradeMode(Enum):
    # 交易模式
    # 保证金模式
    ISOLATED = "isolated"   # 逐仓
    CROSS = "cross" # 全仓
    # 非保证金模式
    CASH = "cash"


class MgnMode(Enum):
    # 保证金模式
    ISOLATED = "isolated"  # 逐仓
    CROSS = "cross"  # 全仓


class Side(Enum):
    BUY = "buy"
    SELL = "sell"
    NET = "net"


class OrderType(Enum):
    MARKET = "market"   # 市价单
    LIMIT = "limit"     # 限价单
    POST_ONLY = "post_only" # 限价委托，只做maker单，如果该笔订单的任何部分会吃掉当前挂单深度，则该订单将被全部撤销。
    FOK = "fok" # 限价委托，全部成交或立即取消，如果无法全部成交该笔订单，则该订单将被全部撤销
    IOC = "ioc" # 限价委托，立即成交并取消剩余，立即按照委托价格成交，并取消该订单剩余未完成数量，不会再深度列表上展示委托数量
    OPTIMAL_LIMIT_IOC = "optimal_limit_ioc" # 市价委托，立即成交并取消剩余，仅适用于交割合约和永续合约


class InstType(Enum):
    # 产品类型
    SPOT = "SPOT"   # 币币
    MARGIN = "MARGIN"   # 币币杠杆
    SWAP = "SWAP"   # 永续合约
    FUTURES = "FUTURES"     # 交割合约
    OPTION = "OPTION"   # 期权
    ANY = "ANY"     # 全部


class OrderState(Enum):
    # 订单状态
    LIVE = "live"   # 等待成交
    PARTIALLY_FILLED = "partially_filled"   # 部分成交
    FILLED = "filled"   # 完全成交
    CANCELED = "canceled"    # 撤单成功


class OrderCategory(Enum):
    # 订单种类
    TWAP = "twap"   # TWAP自动换币
    ADL = "adl" # ADL自动减仓
    FULL_LIQUIDATION = "full_liquidation"   # 强制平仓
    PARTIAL_LIQUIDATION = "partial_liquidation" # 强制减仓
    DELIVERY = "delivery"   # 交割


class PosMode(Enum):
    # 持仓模式
    NET = "net_mode"   # 单向
    LONG_SHORT = "long_short_mode"  # 双向


class PosSide(Enum):
    LONG = "long"
    SHORT = "short"


class SystemState(Enum):
    # 系统升级状态
    SCHEDULED = "scheduled" # 等待中
    ONGOING = "ongoing" # 进行中
    COMPLETED = "completed" # 已完成
    CANCELED = "canceled"   # 已取消


class WebSocketOp(Enum):
    LOGIN = "login"
    SUB = "subscribe"
    UNSUB = "unsubscribe"


class WebSocketEvent:
    ERROR = "error"
    SUB = "subscribe"
    LOGIN = "login"


class KlineType(Enum):
    candle1Y = "candle1Y"
    candle6M = "candle6M"
    candle3M = "candle3M"
    candle1M = "candle1M"
    candle1W = "candle1W"
    candle1D = "candle1D"
    candle2D = "candle2D"
    candle3D = "candle3D"
    candle5D = "candle5D"
    candle12H = "candle12H"
    candle6H = "candle6H"
    candle4H = "candle4H"
    candle2H = "candle2H"
    candle1H = "candle1H"
    candle30m = "candle30m"
    candle15m = "candle15m"
    candle5m = "candle5m"
    candle3m = "candle3m"
    candle1m = "candle1m"


class BooksType(Enum):
    # 深度类型
    books = "books"
    books5 = "books5"
    books_l2_tbt = "books-l2-tbt"
    books50_l2_tbt = "books50-l2-tbt"



import logging
import re
from common.huobi.client import AsyncClient
from common.huobi.constants import TradeType, OrderPriceType
from common.huobi.utils import reverse_direction
from common.huobi.exceptions import HuobiAPIException


class HuobiService:
    def __init__(self, client: AsyncClient):
        self.client = client

    async def close_all_position(self, trade_type, contract_code):
        """
        关闭所有仓位, 使用闪电平仓模式
        """
        position_info = await self.position_info(trade_type, contract_code)
        if trade_type == TradeType.FUTURE:
            func = self.client.future_lightning_close_position
        elif trade_type == TradeType.COIN_M:
            func = self.client.coin_margined_lightning_close_position
        else:
            raise Exception(f"unsupported trade type: {trade_type}")

        for position in position_info:
            params = {
                'contract_code': position['contract_code'],
                'volume': int(position['available']),
                'direction': reverse_direction(position['direction']),
                "order_price_type": OrderPriceType.LIGHTNING
            }
            logging.debug(f"闪电平仓: {params}")
            await func(**params)

    async def position_info(self, trade_type, contract_code) -> list:
        """
        获取当前仓位信息
        """
        if trade_type == TradeType.FUTURE:
            symbol = re.match("^([a-zA-Z]+)[0-9]+$", contract_code).group(1)
            rst = await self.client.future_position_info(symbol)
            data = [e for e in rst.data if e['contract_code'] == contract_code]
        elif trade_type == TradeType.COIN_M:
            rst = await self.client.coin_margined_position_info(contract_code)
            data = [e for e in rst.data if e['contract_code'] == contract_code]
        else:
            raise Exception(f"unsupported trade type: {trade_type}")
        return data

    async def cancel_all_orders(self, trade_type, symbol=None, contract_code=None, contract_type=None, direction=None,
                                offset=None):
        """
        全部撤单
        """
        if trade_type == TradeType.FUTURE:
            func = self.client.future_cancel_all
            if symbol is None and contract_code is not None:
                symbol = re.match("^([a-zA-Z]+)[0-9]+$", contract_code).group(1)
            params = {
                'symbol': symbol,
                'contract_code': contract_code,
                'contract_type': contract_type,
                'direction': direction,
                'offset': offset
            }
        elif trade_type == TradeType.COIN_M:
            func = self.client.coin_margined_cancel_all
            params = {
                'contract_code': contract_code,
                'direction': direction,
                'offset': offset
            }
        else:
            raise Exception(f"unsupported trade type: {trade_type}")
        try:
            return await func(**params)
        except HuobiAPIException as e:
            if e.error_code != 1051:
                raise e

    async def cancel_order(self, trade_type, order_id, symbol=None, contract_code=None):
        params = {"order_id": order_id}
        if trade_type == TradeType.FUTURE:
            func = self.client.future_cancel_order
            params["symbol"] = symbol
        elif trade_type == TradeType.COIN_M:
            func = self.client.coin_margined_cancel_order
            params["contract_code"] = contract_code
        elif trade_type == TradeType.USD_M:
            func = self.client.usd_margined_cancel_order
            params['contract_code'] = contract_code
        else:
            func = self.client.spot_cancel_order
        # Result(
        # code=200,
        # status='ok',
        # msg='',
        # data={'errors': [{'order_id': '872859639523352576', 'err_code': 1063, 'err_msg': 'The order has been executed.'}],
        # 'successes': ''}
        # )
        rst = await func(**params)
        return rst



"""
常量
"""


KLINE_INTERVAL_1MINUTE = '1min'
KLINE_INTERVAL_5MINUTE = '5min'
KLINE_INTERVAL_15MINUTE = '15min'
KLINE_INTERVAL_30MINUTE = '30min'
KLINE_INTERVAL_60MINUTE = '60min'
KLINE_INTERVAL_1HOUR = '1hour'
KLINE_INTERVAL_4HOUR = '4hour'
KLINE_INTERVAL_1DAY = '1day'
KLINE_INTERVAL_1WEEK = '1week'
KLINE_INTERVAL_1MONTH = '1mon'

WS_DEPTH_0 = "step0"
WS_DEPTH_1 = "step1"
WS_DEPTH_2 = "step2"
WS_DEPTH_3 = "step3"
WS_DEPTH_4 = "step4"
WS_DEPTH_5 = "step5"
WS_DEPTH_6 = "step6"
WS_DEPTH_7 = "step7"
WS_DEPTH_8 = "step8"
WS_DEPTH_9 = "step9"
WS_DEPTH_10 = "step10"
WS_DEPTH_11 = "step11"
WS_DEPTH_12 = "step12"
WS_DEPTH_13 = "step13"
WS_DEPTH_14 = "step14"
WS_DEPTH_15 = "step15"
WS_DEPTH_16 = "step16"
WS_DEPTH_17 = "step17"
WS_DEPTH_18 = "step18"
WS_DEPTH_19 = "step19"


class DataType:
    INCREMENTAL = "incremental"
    SNAPSHOT = "snapshot"


class WebSocketOP:
    SUB = "sub"
    REQ = "req"
    UNSUB = "unsub"
    AUTH = "auth"
    PING = "ping"
    PONG = "pong"
    NOTIFY = "notify"
    ERROR = "error"
    CLOSE = "close"


class BasisPriceType:
    OPEN = "open"
    CLOSE = "close"
    HIGH = "high"
    LOW = "low"
    AVERAGE = "average"


class AssetChangeEvent:
    """
    资产变化通知相关事件说明，比如订单创建开仓(order.open) 、订单成交(order.match)（除开强平和结算交割）、结算交割(settlement)、
    订单强平成交(order.liquidation)（对钆和接管仓位）、订单撤销(order.cancel) 、合约账户划转（contract.transfer)（包括外部划转）、
    系统（contract.system)、其他资产变化(other)、切换杠杆（switch_lever_rate）、初始资金（init）
    """
    OPEN = "order.open"
    MATCH = "order.match"
    SETTLEMENT = "settlement"
    LIQUIDATION = "order.liquidation"
    CANCEL = "order.cancel"
    TRANSFER = "contract.transfer"
    SYSTEM = "contract.system"
    OTHER = "other"
    SWITCH_LEVER_RATE = "switch_lever_rate"
    INIT = "init"


class ContractType:
    THIS_WEEK = "this_week"
    NEXT_WEEK = "next_week"
    QUARTER = "quarter"
    NEXT_QUARTER = "next_quarter"


class TradeType:
    SPOT = "spot"
    FUTURE = "future"
    COIN_M = "coin_margined"
    USD_M = "usdt_margined"


class WSMsgFormat:
    SIMPLE = 'simple'   # {'sub': 'xxx'}
    STANDARD = 'standard'   # {'op': 'sub', 'topic': 'xxx'}


class OrderStatus:
    ReadyToSubmit1 = 1  # 准备提交
    ReadyToSubmit2 = 2  # 2准备提交
    Submitted = 3   # 已提交
    PartiallyMatched = 4    # 部分成交
    CancelledWithPartiallyMatched = 5   # 部分成交已撤单
    FullyMatched = 6    # 全部成交
    Cancelled = 7   # 已撤单
    Cancelling = 11  # 撤单中


class OrderPriceType:
    LIGHTNING = 'lightning'
    LIMIT = 'limit'
    POST_ONLY = 'post_only'     # 只做maker单
    OPPONENT = 'opponent'   # 对手价
    FOK = 'fok'


class Side:
    BUY = "buy"
    SELL = "sell"







ERROR_CODE = {
    # 通用
    0: "操作成功",
    1: "操作全部失败",
    2: "批量操作部分成功",
    50000: "body不能为空",
    50001: "撮合引擎正在升级，请稍后重试",
    50002: "非法的json数据",
    50004: "接口请求超时（不代表请求成功或者失败，请检查请求结果）",
    50005: "接口已下线或无法使用",
    50006: "无效的Content_Type，请使用application/json格式",
    50007: "用户被冻结",
    50008: "用户不存在",
    50009: "用户处于爆仓冻结",
    50010: "用户ID为空",
    50011: "用户请求频率过快，超过该接口允许的限额",
    50012: "用户状态无效",
    50013: "当前系统繁忙，请稍后重试",
    50014: "必填参数{0}不能为空",
    50015: "参数{0}和{1}不能同时为空",
    50016: "参数{0}和{1}不匹配",
    50017: "当前仓位处于ADL冻结中，无法进行相关操作",
    50018: "{0}币种处于ADL冻结中，无法进行相关操作",
    50019: "当前账户处于ADL冻结中，无法进行相关操作",
    50020: "当前仓位处于强平冻结中，无法进行相关操作",
    50021: "{0}币种处于强平冻结中，无法进行相关操作",
    50022: "当前账户处于强平冻结中，无法进行相关操作",
    50023: "资金费冻结，无法进行相关操作",
    50024: "参数{0}和{1}不能同时存在",
    50025: "参数{0}传值个数超过最大限制{1}",
    50026: "系统错误",
    50027: "当前账户已被限制交易",
    50028: "账户异常无法下单",
    50029: "你的账户已经触发风控体系，禁止该标的{0}交易，请联系客服进行处理",
    # API 类
    # 交易类
    51023: "仓位不存在"
}


import asyncio
import logging

import pytest
from common.okex.stream import ReconnectingWebSocket
from common.okex.constants import *


@pytest.mark.asyncio
async def test_public_websocket(logger, okex_pub_ws: ReconnectingWebSocket):
    async with okex_pub_ws as ws:
        await ws.sub({
            "channel": "tickers",
            "instId": "BTC-USDT"
        })
        while True:
            msg = await ws.recv()
            logger.info(msg)


@pytest.mark.asyncio
async def test_private_websocket(logger, okex_pri_ws: ReconnectingWebSocket):
    async with okex_pri_ws as ws:
        await ws.sub({
            "channel": "orders",
            "instType": "FUTURES",
            "instId": "BTC-USD-210730"
        })
        while True:
            msg = await ws.recv()
            logger.info(msg)





import pytest
from common.models import Result
from common.huobi.service import HuobiService
from common.huobi.constants import TradeType
from common.huobi.exceptions import HuobiAPIException


@pytest.mark.asyncio
async def test_cancel_all_orders(huobi_service: HuobiService, logger):
    try:
        resp: Result = await huobi_service.cancel_all_orders(TradeType.FUTURE, symbol='fil')
        logger.info(resp)
    except HuobiAPIException as e:
        if e.error_code != 1051:
            raise e

    try:
        resp: Result = await huobi_service.cancel_all_orders(TradeType.COIN_M, contract_code='fil-usd')
        logger.info(resp)
    except HuobiAPIException as e:
        if e.error_code != 1051:
            raise e




from common.binance.check import *
from .config import Binance


client = Client(Binance.API_KEY, Binance.API_SECRET)


def test_check_system_status():
    assert check_system_status(client) is True


def test_check_account_status():
    assert check_account_status(client) is True


def test_check_api_trading_status():
    assert check_api_trading_status(client) is True


def test_check_wallet_can_trade():
    assert check_can_trade(client) is True






import gzip
import json
import logging
import asyncio
from functools import partial
from typing import Optional
import aiohttp
import time
from aiohttp.web_ws import WSMessage

from common.exceptions import OkexWebSocketError
from common.okex.tools import generate_signature
from common.okex.constants import *
from common.dict_util import remove_key


class ReconnectingWebSocket:
    # 如果连接成功后30s未订阅或订阅后30s内服务器未向用户推送数据，系统会自动断开连接
    TIMEOUT = 20

    # 实盘
    WS_PUBLIC_URL = "wss://ws.okex.com:8443/ws/v5/public"
    WS_PRIVATE_URL = "wss://ws.okex.com:8443/ws/v5/private"
    # 模拟盘
    SIMULATED_WS_PUBLIC_URL = "wss://ws.okex.com:8443/ws/v5/public?brokerId=9999"
    SIMULATED_WS_PRIVATE_URL = "wss://wspap.okex.com:8443/ws/v5/private?brokerId=9999"

    def __init__(self, public=True, proxy=None, api_key: Optional[str] = None, api_secret: Optional[str] = None,
                 api_passphrase: Optional[str] = None, simulated=False):
        self._public = public
        self.ws: Optional[aiohttp.ClientWebSocketResponse] = None
        self._session: Optional[aiohttp.ClientSession] = None
        self._conn = None
        self._proxy = proxy
        self._api_key = api_key
        self._api_secret = api_secret
        self._api_passphrase = api_passphrase
        self._connected = False

        # 用于断开重连时再次订阅
        self._sub_channel_msg = []

        # 以下认证相关变量
        self._already_send_auth = False
        # 确保login消息处理完毕
        self._complete_auth = False
        # 通知事件等待者已完成认证
        self.auth_complete_event = asyncio.Event()

        self._simulated = simulated

    @property
    def url(self) -> str:
        if self._simulated:
            return self.SIMULATED_WS_PUBLIC_URL if self._public else self.SIMULATED_WS_PRIVATE_URL
        else:
            return self.WS_PUBLIC_URL if self._public else self.WS_PRIVATE_URL

    @property
    def need_send_auth(self) -> bool:
        return not self._public and not self._already_send_auth

    @property
    def need_reconnected(self) -> bool:
        return self.ws is None or self.ws.closed or not self._connected

    @property
    def api_key(self):
        return self._api_key

    async def __aenter__(self):
        if self.need_reconnected:
            self._reset_flag()
            await self._reconnect()
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        if self.ws:
            await self.ws.close()
            self.ws = None
        if self._conn:
            await self._conn.__aexit(exc_type, exc_val, exc_tb)
        self._reset_flag()
        if self._session:
            await self._session.close()
            self._session = None

    def _package_msg(self, op: WebSocketOp, payload):
        msg = {"op": op.value}
        if isinstance(payload, dict):
            msg['args'] = [payload]
        else:
            msg['args'] = payload
        for item in msg['args']:
            for k, v in item.items():
                if isinstance(v, Enum):
                    item[k] = v.value
        return msg

    def _reset_flag(self):
        """
        连接断开后重置标志变量
        """
        # 断开后conn已经失效，必须重连
        self._conn = None
        self._connected = self._already_send_auth = self._complete_auth = False

    async def send_auth(self):
        if not self.ws:
            raise OkexWebSocketError("no binding web socket")
        if self.ws.closed:
            raise OkexWebSocketError("websocket has closed")

        timestamp = str(int(time.time()))
        payload = {
            "apiKey": self._api_key,
            "passphrase": self._api_passphrase,
            "timestamp": timestamp,
            "sign": generate_signature("GET", "/users/self/verify", timestamp, None, None, self._api_secret)
        }
        logging.debug(f"send auth msg: {payload}")
        await self.ws.send_json(self._package_msg(WebSocketOp.LOGIN, payload))
        self._already_send_auth = True

    async def _connect(self):
        logging.info(f"ws connect {self.url}")
        if not self._session or self._session.closed:
            self._session = aiohttp.ClientSession(trust_env=True)
            self._conn = None
            self.ws = None
        if not self._conn:
            self._conn = self._session.ws_connect(self.url, proxy=self._proxy)
            self.ws = None
        try:
            if not self.ws:
                self.ws = await self._conn.__aenter__()
            self._connected = True
            logging.debug(f"connect {self.url} successfully!")
        except aiohttp.client.ClientConnectorError:
            logging.error(f"connect to ws server error! url: {self.url}")

    async def _connected_callback(self):
        if self.need_send_auth:
            await self.send_auth()
            logging.debug("wait for auth response")
            while self._complete_auth is False:
                await self.recv()
            self.auth_complete_event.set()
            logging.debug("auth complete!")
        if self._sub_channel_msg:
            msg = self._package_msg(WebSocketOp.SUB, self._sub_channel_msg)
            await self.ws.send_json(msg)

    async def _reconnect(self):
        logging.warning("reconnecting web socket right now!")
        await self._connect()
        await self._connected_callback()

    async def recv(self):
        res = None
        if self.ws is None:
            raise OkexWebSocketError("web socket connection not connected yet")
        while not res:
            if self.ws.closed:
                self._reset_flag()
                await self._reconnect()
                continue
            try:
                res = await asyncio.wait_for(self.ws.receive(), timeout=self.TIMEOUT)
            except asyncio.TimeoutError:
                logging.error(f"no message in {self.TIMEOUT} seconds")
            except asyncio.CancelledError as e:
                logging.error(f"cancelled error {e}")
            except asyncio.IncompleteReadError as e:
                logging.error(f"incomplete read error {e}")
            except Exception as e:
                logging.error(f"exception {e}")
            else:
                res = self._handle_message(res)

            if isinstance(res, dict):
                if res.get("event") == WebSocketEvent.LOGIN and res.get("code") == '0':
                    self._complete_auth = True
            elif res == "pong":
                res = None

        return res

    def _handle_message(self, msg: WSMessage):
        if msg.type == aiohttp.WSMsgType.TEXT:
            try:
                data = json.loads(msg.data)
            except:
                data = msg.data
        elif msg.type == aiohttp.WSMsgType.BINARY:
            uncompress_data = gzip.decompress(msg.data).decode()
            try:
                data = json.loads(uncompress_data)
            except:
                data = uncompress_data
        elif msg.type == aiohttp.WSMsgType.CLOSE:
            data = None
        else:
            raise OkexWebSocketError(f"invalid msg type: {msg.type}")

        logging.debug(f"receive msg: {data}")
        return data

    async def send_request(self, payload):
        if self._connected is False or self.ws is None or self.ws.closed:
            await self._connect()
            await self._connected_callback()
        logging.debug(f"send msg: {payload}")
        await self.ws.send_json(payload)

    async def sub(self, payload):
        msg = self._package_msg(WebSocketOp.SUB, payload)
        if msg not in self._sub_channel_msg:
            self._sub_channel_msg.append(msg)
        await self.send_request(msg)

    async def unsub(self, payload):
        msg = self._package_msg(WebSocketOp.UNSUB, payload)
        if msg in self._sub_channel_msg:
            self._sub_channel_msg.remove(msg)
        await self.send_request(msg)

    async def sub_or_unsub(self, payload, op: WebSocketOp):
        if op == WebSocketOp.SUB:
            await self.sub(payload)
        elif op == WebSocketOp.UNSUB:
            await self.unsub(payload)
        else:
            raise Exception(f"unsupported op: {op.value}")


class OkexSocketManager:
    def __init__(self, proxy=None, api_key: Optional[str] = None, api_secret: Optional[str] = None,
                 api_passphrase: Optional[str] = None):
        self.ws: Optional[ReconnectingWebSocket] = None
        self._create_ws = partial(
            ReconnectingWebSocket, proxy=proxy, api_key=api_key, api_secret=api_secret, api_passphrase=api_passphrase)

    def set_public_simulated_socket(self) -> "OkexSocketManager":
        self.ws = self._create_ws(public=True, simulated=True)
        return self

    def set_private_simulated_socket(self) -> "OkexSocketManager":
        self.ws = self._create_ws(public=False, simulated=True)
        return self

    def set_public_socket(self) -> "OkexSocketManager":
        self.ws = self._create_ws(public=True, simulated=False)
        return self

    def set_private_socket(self) -> "OkexSocketManager":
        self.ws = self._create_ws(public=False, simulated=False)
        return self

    ######################################################
    # 公共频道                                             #
    ######################################################
    async def sub_instruments(self, instType: InstType, op=WebSocketOp.SUB):
        # 产品频道
        # 首次订阅推送产品的全量数据；后续当有产品状态变化时（如期货交割、期权行权、新合约/币对上线、人工暂停/恢复交易等），推送产品的增量数据。
        await self.ws.sub_or_unsub({
            "channel": "instruments",
            "instType": instType
        }, op)

    async def sub_tickers(self, instId: str, op=WebSocketOp.SUB):
        # 行情频道
        # 获取产品的最新成交价、买一价、卖一价和24小时交易量等信息，每100ms有数据更新推送一次数据
        await self.ws.sub_or_unsub({
            "channel": "tickers",
            "instId": instId
        }, op)

    async def sub_open_interest(self, instId: str, op=WebSocketOp.SUB):
        """
        持仓总量频道
        获取持仓总量，每3s有数据更新推送一次数据
        """
        await self.ws.sub_or_unsub({
            "channel": "open-interest",
            "instId": instId
        }, op)

    async def sub_kline(self, kline_type: KlineType, instId: str, op=WebSocketOp.SUB):
        """
        K线频道
        获取产品的K线数据，每500ms推送一次数据。
        """
        await self.ws.sub_or_unsub({
            "channel": kline_type.value,
            "instId": instId
        }, op)

    async def sub_trades(self, instId: str, op=WebSocketOp.SUB):
        """
        交易频道
        获取最近的成交数据，有成交数据就推送
        """
        await self.ws.sub_or_unsub({
            "channel": "trades",
            "instId": instId
        }, op)

    async def sub_estimated_price(self, instType: InstType, uly=None, instId=None, op=WebSocketOp.SUB):
        """
        预估交割/行权价格频道
        获取交割合约和期权预估交割/行权价。交割/行权预估价只有交割/行权前一小时开始推送预估交割/行权价，有价格变化就推送
        uly和instId必须指定一个
        """
        assert instType in (InstType.FUTURES, InstType.OPTION)
        assert not (uly is None and instId is None)
        payload = remove_key(locals(), "self,op")
        payload['channel'] = "estimated-price"
        await self.ws.sub_or_unsub(payload, op)

    async def sub_mark_price(self, instId: str, op=WebSocketOp.SUB):
        """
        标记价格频道
        获取标记价格，标记价格有变化时，每200ms推送一次数据，标记价格没变化时，每10s推送一次数据
        """
        await self.ws.sub_or_unsub({
            "channel": "mark-price",
            "instId": instId
        }, op)

    async def sub_mark_price_kline(self, kline_type: KlineType, instId: str, op=WebSocketOp.SUB):
        """
        标记价格K线频道
        获取标记价格的K线数据，每500ms推送一次数据
        """
        await self.ws.sub_or_unsub({
            "channel": f"mark-price-{kline_type.value}",
            "instId": instId
        }, op)

    async def sub_price_limit(self, instId: str, op=WebSocketOp.SUB):
        """
        限价频道
        获取交易的最高买价和最低卖价。限价有变化时，每5秒推送一次数据，限价没变化时，不推送数据
        """
        await self.ws.sub_or_unsub({
            "channel": "price-limit",
            "instId": instId
        }, op)

    async def sub_books(self, books_type: BooksType, instId: str, op=WebSocketOp.SUB):
        """
        深度频道
        获取深度数据，books是400档频道，books5是5档频道，books-l2-tbt是先400档后实时推送的频道，books50-l2-tbt是先50档后实时推的频道；
        books 首次推400档快照数据，以后增量推送，即每100毫秒有深度变化推送一次变化的数据
        books5首次推5档快照数据，以后定量推送，每100毫秒有深度变化推送一次5档数据，即每次都推送5档数据
        books-l2-tbt 首次推400档快照数据，以后增量推送，即有深度有变化推送一次变化的数据
        books50-l2-tbt 首次推50档快照数据，以后增量推送，即有深度有变化推送一次变化的数据
        """
        await self.ws.sub_or_unsub({
            "channel": books_type.value,
            "instId": instId
        }, op)

    async def sub_opt_summary(self, uly: str, op=WebSocketOp.SUB):
        """
        期权定价频道
        获取所有期权合约详细定价信息，一次性推送所有
        """
        await self.ws.sub_or_unsub({
            "channel": "opt-summary",
            "uly": uly
        }, op)

    async def sub_funding_rate(self, instId: str, op=WebSocketOp.SUB):
        """
        资金费率频道
        获取合约资金费率，一分钟推送一次数据
        """
        await self.ws.sub_or_unsub({
            "channel": "funding-rate",
            "instId": instId
        }, op)

    async def sub_index_kline(self, kline_type: KlineType, instId: str, op=WebSocketOp.SUB):
        """
        指数K线频道
        获取指数的K线数据，每500ms推送一次数据。
        """
        await self.ws.sub_or_unsub({
            "channel": f"index-{kline_type.value}",
            "instId": instId
        }, op)

    async def sub_index_tickers(self, instId: str, op=WebSocketOp.SUB):
        """
        指数行情频道
        获取指数的行情数据
        """
        await self.ws.sub_or_unsub({
            "channel": "index-tickers",
            "instId": instId
        }, op)

    async def sub_status(self, op=WebSocketOp.SUB):
        """
        Status 频道
        获取系统维护的状态，当系统维护状态改变时推送。首次订阅：”推送最新一条的变化数据“；后续每次有状态变化，推送变化的内容
        """
        await self.ws.sub_or_unsub({
            "channel": "status"
        }, op)

    ######################################################
    # 私有频道                                             #
    ######################################################
    async def sub_account(self, ccy: str, op=WebSocketOp.SUB):
        """
        账户频道
        获取账户信息，首次订阅按照订阅维度推送数据，此外，当下单、撤单等事件触发时，推送数据以及按照订阅维度定时推送数据
        """
        await self.ws.sub_or_unsub({
            "channel": "account",
            "ccy": ccy
        }, op)

    async def sub_positions(self, instType: InstType, uly: str = None, instId: str = None, op=WebSocketOp.SUB):
        """
        持仓频道
        获取持仓信息，首次订阅按照订阅维度推送数据，此外，当下单、撤单等事件触发时，推送数据以及按照订阅维度定时推送数据
        """
        assert not (uly is None and instId is None)
        payload = remove_key(locals(), "self,op")
        payload['channel'] = "positions"
        await self.ws.sub_or_unsub(payload, op)

    async def sub_balance_and_position(self, op=WebSocketOp.SUB):
        """
        账户余额和持仓频道
        获取账户余额和持仓信息，首次订阅按照订阅维度推送数据，此外，当成交、资金划转等事件触发时，推送数据。
        该频道适用于尽快获取账户现金余额和仓位资产变化的信息。
        """
        await self.ws.sub_or_unsub({
            "channel": "balance_and_position",
        }, op)

    async def sub_orders(self, instType: InstType, uly=None, instId=None, op=WebSocketOp.SUB):
        """
        订单频道
        获取订单信息，首次订阅不推送，只有当下单、撤单等事件触发时，推送数据
        """
        payload = remove_key(locals(), "self,op")
        payload['channel'] = "orders"
        await self.ws.sub_or_unsub(payload, op)

    async def sub_orders_algo(self, instType: InstType, uly=None, instId=None, op=WebSocketOp.SUB):
        """
        策略委托订单频道
        获取策略委托订单，首次订阅不推送，只有当下单、撤单等事件触发时，推送数据
        """
        assert instType != instType.OPTION
        payload = remove_key(locals(), "self,op")
        payload['channel'] = "orders-algo"
        await self.ws.sub_or_unsub(payload, op)









import logging
from logging import Logger
import sys


logging_pool = dict()


def create_logger(name: str = "root", file_path="") -> Logger:
    if name in logging_pool:
        return logging_pool[name]

    logger = logging.getLogger(name)
    logger.setLevel(logging.DEBUG)

    formatter = logging.Formatter(
        fmt="%(asctime)s %(name)s (%(levelname)s) - [%(filename)s:%(lineno)d]: %(message)s",
        datefmt='%y-%m-%d %H:%M:%S'
    )

    if file_path:
        fh = logging.FileHandler(file_path)
        fh.setLevel(logging.DEBUG)
        fh.setFormatter(formatter)
        logger.addHandler(fh)

    ch = logging.StreamHandler(sys.stdout)
    ch.setLevel(logging.INFO)
    ch.setFormatter(formatter)

    logger.addHandler(ch)
    logging_pool[name] = logger
    return logger



from common.huobi.wallet import Wallet
from common.log import create_logger
from .config import Huobi

wallet = Wallet(Huobi.API_KEY, Huobi.API_SECRET)
logger = create_logger("pytest")


def test_spot_balance():
    for item in wallet.get_spot_balance().values():
        item.print_object()


################
# 币本位
################
def test_coin_margined_swap_balance_valuation():
    resp = wallet.get_coin_margined_swap_balance_valuation("USD")
    logger.info(resp)


def test_coin_margined_swap_account_info():
    print(wallet.get_coin_margined_swap_account_info())


def test_coin_margined_swap_position_info():
    print(wallet.get_coin_margined_swap_position())


################
# 交割合约
################
def test_get_contract_account_info():
    print(wallet.get_contract_account_info())


def test_get_contract_position_info():
    print(wallet.get_contract_position_info())



from common.okex.client import AsyncClient


class OkexService:
    def __init__(self, client: AsyncClient):
        self.client = client

    async def cancel_all_orders(self, instId_list: [str]):
        """
        全部撤单
        """
        rst = await self.client.pending_orders()
        orders = rst.data
        payload = [{
            "ordId": order['ordId'],
            "instId": order['instId']
        } for order in orders if order['instId'] in instId_list]

        if payload:
            await self.client.batch_cancel_orders(payload)



import re

from common.huobi.client import AsyncClient
from common.huobi.constants import TradeType
from common.models import Result


class AccountService:
    def __init__(self, client: AsyncClient):
        self.client = client

    async def account_info(self, trade_type, symbol: str):
        # symbol 可能有如下格式: btc-usd,btc_nw,btc210420
        if "_" in symbol:
            symbol = symbol.split('_')[0]
        else:
            match_value = re.match('^([a-zA-Z]+)([0-9]+)$', symbol)
            if match_value:
                symbol = match_value.groups()[0]

        rst: Result
        if trade_type == TradeType.FUTURE:
            # symbol: btc,eth,...
            rst = await self.client.future_account_info(symbol)
        elif trade_type == TradeType.COIN_M:
            # symbol: btc-usd, eth-usd,...
            rst = await self.client.coin_margined_account_info(contract_code=symbol)
        else:
            raise Exception(f"unsupported trade type: {trade_type}")
        return rst.data

    async def transfer(self, symbol, from_type, to_type, amount):
        """
        资金划转
        如果不涉及币币账户，都需要币币作为中转
        """
        assert from_type != to_type

        if from_type == TradeType.SPOT:
            if to_type == TradeType.FUTURE:
                return await self.client.future_transfer(symbol, amount, transfer_type="pro-to-futures")
            elif to_type == TradeType.COIN_M:
                return await self.client.coin_margined_transfer("spot", "swap", symbol, amount)
        elif from_type == TradeType.FUTURE:
            if to_type == TradeType.SPOT:
                return await self.client.future_transfer(symbol, amount, transfer_type="futures-to-pro")
            elif to_type == TradeType.COIN_M:
                await self.transfer(symbol, from_type, TradeType.SPOT, amount)
                return await self.transfer(symbol, TradeType.SPOT, to_type, amount)
        elif from_type == TradeType.COIN_M:
            if to_type == TradeType.SPOT:
                return await self.client.coin_margined_transfer("swap", "spot", symbol, amount)
            elif to_type == TradeType.FUTURE:
                await self.transfer(symbol, from_type, TradeType.SPOT, amount)
                return await self.transfer(symbol, TradeType.SPOT, to_type, amount)





import pytest
from common.okex.client import AsyncClient
from common.okex.constants import *


@pytest.mark.asyncio
async def test_account_balance(okex_client: AsyncClient, logger):
    resp = await okex_client.account_balance("BTC")
    logger.info(resp)


@pytest.mark.asyncio
async def test_account_position(okex_client: AsyncClient, logger):
    resp = await okex_client.positions(instId="BTC-USD-SWAP")
    logger.info(resp)


@pytest.mark.asyncio
async def test_create_order(okex_client: AsyncClient, logger):
    resp = await okex_client.create_order(
        "BTC-USDT",
        TradeMode.CASH,
        Side.BUY,
        OrderType.MARKET,
        sz=str(100)
    )
    logger.info(resp)



from setuptools import setup, find_packages

setup(
    name="gecko-common",
    version="0.1.84",
    author="sgy",
    author_email="ericsgy@163.com",
    description="gecko common library",
    url="https://gitlab.com/winwin2021/common",
    license='MIT',
    keywords='binance huobi ok exchange',

    # 你要安装的包，通过 setuptools.find_packages 找到当前目录下有哪些包
    packages=find_packages(exclude=('tests',)),

    # 分类信息
    classifiers=[
        # 发展时期
        'Development Status :: 3 - Alpha',
        # 开发的目标用户
        'Intended Audience :: Developers',
        # 类型
        'Topic :: Software Development :: Libraries :: Python Modules',
        # 许可证信息
        'License :: OSI Approved :: MIT License',
        # 目标 Python 版本
        'Programming Language :: Python :: 3.7'
    ],

    # 表明当前模块依赖哪些包，若环境中没有，则会从pypi中下载安装
    install_requires=[
        'aiohttp >= 3.6.2, <=4',
        'python-binance2',
    ],
    # 仅在测试时需要使用的依赖，在正常发布的代码中是没有用的
    # 在执行python setup.py test时，可以自动这些库，确保测试的正常运行
    test_requires=[
        'requests >=2.25.1, <=3',
        'requests-mock >=1.9.2, <=2',
        'pytest >=6.2.4, <=7',
        'pytest-asyncio >= 0.15.1'
    ],

    python_requires='>=3.7',
)
